﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace CS_Magnus
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }
        //-------------------------------------------------------------
        static int NUM_MAX = 1001;//データ個数最大値
        int numMax;//実際のデータ個数
        int drawCount;
        int dataNo;
        double omegaX;
        float[] posY = new float[NUM_MAX];
        float[] posZ = new float[NUM_MAX];

        Graphics g;

        private void Form1_Load(object sender, EventArgs e)
        {
            txtHeight.Text = "5.0";
            txtDiameter.Text = "0.74"; //野球ボール(見かけの大きさは10倍）
            txtMass.Text = "0.145";
            txtVelocity.Text = "120.0";//[km/h]
            txtTheta.Text = "0.0";   //[deg]
            txtOmegaX1.Text = "0.0";   //[rps]
            txtOmegaX2.Text = "15.0";  //[rps]
            txtOmegaX3.Text = "30.0";  //[rps]
            txtOmegaX4.Text = "40.0";  //[rps]
            txtScale.Text = "50.0";   //[m]
            txtTimeStep.Text = "0.001";//[s]
            txtInterval.Text = "0.1";  //[s]
            timer1.Enabled = false;

            radNon.Checked = true;
            radInertial.Checked = false;
            chkLine.Checked = false;
            chkStrobo.Checked = true;

            drawScale();
        }

        private void drawScale()
        {
            float x0, y0;
            picSpace.Image = new Bitmap(picSpace.Width, picSpace.Height);
            g = Graphics.FromImage(picSpace.Image);
            g.Clear(Color.White);

            double scale = Convert.ToDouble(txtScale.Text);
            if (scale == 0.0) return;
            double scaleM = 0.7 * picSpace.Height / scale;//主目盛間隔

            //左端位置
            x0 = 50.0f;
            y0 = (float)(picSpace.Height * 0.9f);

            g = Graphics.FromImage(picSpace.Image);
            Pen pen = new Pen(Color.Blue, 1);
            //縦軸
            g.DrawLine(pen, x0, y0 * 1.05f, x0, y0 * 0.05f);
            //横軸
            g.DrawLine(pen, x0, y0, picSpace.Width * 0.95f, y0);
            //縦軸目盛
            g.DrawLine(pen, x0, y0 - (float)(scale * scaleM), x0 - 5.0f, y0 - (float)(scale * scaleM));
            g.DrawString(Convert.ToString(scale), new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 40, y0 - (float)(scale * scaleM + 10.0));
            g.DrawString("0", new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 20, y0 - 10);
            //横軸目盛
            g.DrawLine(pen, x0 + (float)(scale * scaleM), y0, x0 + (float)(scale * scaleM), y0 - 20);
            g.DrawLine(pen, x0 + (float)(scale * scaleM) * 2.0f, y0, x0 + (float)(scale * scaleM) * 2.0f, y0 - 20);
            g.DrawString(Convert.ToString(scale), new Font("Century", 10), new SolidBrush(Color.Blue), x0 + (float)(scale * scaleM) - 15, y0 + 7);
            g.DrawString(Convert.ToString(2.0 * scale), new Font("Century", 10), new SolidBrush(Color.Blue), x0 + (float)(scale * scaleM) * 2 - 15, y0 + 7);

            g.Dispose();
            picSpace.Invalidate();
        }

        private void btnStart1_Click(object sender, EventArgs e)
        {
            omegaX = Convert.ToDouble(txtOmegaX1.Text) * Math.PI * 2.0;

            dataNo = 1;
            calcPos();
        }

        private void btnStart2_Click(object sender, EventArgs e)
        {
            omegaX = Convert.ToDouble(txtOmegaX2.Text) * Math.PI * 2.0;

            dataNo = 2;
            calcPos();
        }

        private void btnStart3_Click(object sender, EventArgs e)
        {
            omegaX = Convert.ToDouble(txtOmegaX3.Text) * Math.PI * 2.0;

            dataNo = 3;
            calcPos();
        }

        private void btnStart4_Click(object sender, EventArgs e)
        {
            omegaX = Convert.ToDouble(txtOmegaX4.Text) * Math.PI * 2.0;

            dataNo = 4;
            calcPos();
        }
        private void btnClear_Click(object sender, EventArgs e)
        {
            g = Graphics.FromImage(picSpace.Image);
            g.Clear(Color.White);
            drawScale();
            g.Dispose();
            picSpace.Invalidate();

            listBox1.Items.Clear();
        }

        //-------------------------------------------------------------- 
        private void calcPos()
        {
            double height = Convert.ToDouble(txtHeight.Text);
            double dia = Convert.ToDouble(txtDiameter.Text);
            double mass = Convert.ToDouble(txtMass.Text);
            double velocity = Convert.ToDouble(txtVelocity.Text) * 1000.0 / 3600; //[Km/h]->[m/s]に変換
            double theta = Convert.ToDouble(txtTheta.Text);
            double timeStep = Convert.ToDouble(txtTimeStep.Text);
            double timeInterval = Convert.ToDouble(txtInterval.Text);
            double maxHeight = 0.0;
            double maxDistance = 0.0;
            double maxTime = 0.0;//地面に落下したときの時間

            double gravity = 9.8;

            //空気に対する慣性抵抗係数
            double rho = 1.2;//空気密度（20°C)
            double A = Math.PI * dia * dia　 / 4.0;//球断面積
            double Ci = 0.25 * rho * A *0.01;//見かけの面積の1/100
            double gammaI = Ci / mass;
            //MessageBox.Show("gammaV[50] = " + gammaV.ToString() + ",  gammaI = " + gammaI.ToString());

            //マグナス効果の比例係数
            double kMagnus = Math.PI * rho * dia * dia * dia * 0.001 / 8.0;//見かけの体積の1/1000

            //初期値
            double y = 0.0;
            double z = height;
            double yb = 0.0;   //到達距離を求めるとき使用
            double zb = height;//到達距離を求めるとき使用
            double velY = velocity * Math.Cos(Math.PI * theta / 180.0);
            double velZ = velocity * Math.Sin(Math.PI * theta / 180.0);
            
            timer1.Enabled = false;
            //オイラー法
            posY[0] = (float)y;
            posZ[0] = (float)z;
            int numCount = 0;
            double timeCount = 0.0;
            double tt = 0.0;//経過時間
            while (numCount < NUM_MAX - 1)
            {
                if (radNon.Checked)
                {
                    yb = y; zb = z;
                    velY += -kMagnus * omegaX * velZ * timeStep / mass;
                    y += velY * timeStep;
                    velZ += (kMagnus * omegaX * velY / mass - gravity) * timeStep;
                    z += velZ * timeStep;
                    timeCount += timeStep;
                    if (z > maxHeight) maxHeight = z;//最高点
                    if (zb >= 0.0 && z <= 0.0)
                    {
                        maxDistance = (y + yb) / 2.0;//到達距離
                        maxTime = tt;
                    }
                    if (timeCount >= timeInterval)
                    {
                        numCount++;
                        if (numCount >= NUM_MAX) break;
                        posY[numCount] = (float)y;
                        posZ[numCount] = (float)z;
                        timeCount = 0.0;
                        numMax = numCount;
                        if (z < -1.0) break;
                    }
                }
                else if (radInertial.Checked)
                {
                    yb = y; zb = z;
                    double speed = Math.Sqrt(velY * velY + velZ * velZ);
                    velY += -(kMagnus * omegaX * velZ + gammaI * speed * velY) * timeStep;
                    y += velY * timeStep;
                    velZ += (kMagnus * omegaX * velY / mass - gravity - gammaI * speed * velZ) * timeStep;
                    z += velZ * timeStep;
                    timeCount += timeStep;
                    if (z > maxHeight) maxHeight = z;//最高点
                    if (zb >= 0.0 && z <= 0.0)
                    {
                        maxDistance = (y + yb) / 2.0;//到達距離
                        maxTime = tt;
                    }
                    if (timeCount >= timeInterval)
                    {
                        numCount++;
                        if (numCount >= NUM_MAX) break;
                        posY[numCount] = (float)y;
                        posZ[numCount] = (float)z;
                        timeCount = 0.0;
                        numMax = numCount;
                        if (z < -1.0) break;
                    }
                }
            }
            drawCount = 0;
            if (chkStrobo.Checked)
            {
                timer1.Interval = (int)(1000.0 * timeInterval);//ms
                timer1.Enabled = true;
            }
            if (chkLine.Checked) drawLine(posY, posZ, numMax, dataNo);

            //MessageBox.Show("numMax = " + numMax.ToString() + "z = " + z.ToString());

            listBox1.Items.Add("  ");
            listBox1.Items.Add("  No = " + dataNo.ToString());
            listBox1.Items.Add("    H = " + maxHeight.ToString("0.000") + "m");
            listBox1.Items.Add("    D = " + maxDistance.ToString("0.000") + "m");

        }
        //--------------------------------------------------------------
        private void drawLine(float[] posY, float[] posZ, int numMax, int dataNo)
        {
            int i;
            float x0, y0, x1, y1, x2, y2;
            float scale = Convert.ToSingle(txtScale.Text);
            float scaleM = 0.7f * picSpace.Height / scale;//主目盛間隔
            //MessageBox.Show("scale = " + scale.ToString() + ",  scaleM = " + scaleM.ToString());

            //左端位置
            x0 = 50.0f;
            y0 = (float)(picSpace.Height * 0.9f);

            g = Graphics.FromImage(picSpace.Image);

            //描画
            for (i = 0; i <= numMax; i++)
            {
                x1 = x0 + posY[0] * scaleM;
                y1 = y0 - posZ[0] * scaleM;
                for (i = 1; i < numMax; i++)
                {
                    x2 = x0 + posY[i] * scaleM;
                    y2 = y0 - posZ[i] * scaleM;
                    if (dataNo == 1)
                        g.DrawLine(new Pen(Color.Black, 2), x1, y1, x2, y2);
                    else if (dataNo == 2)
                        g.DrawLine(new Pen(Color.Red, 2), x1, y1, x2, y2);
                    else if (dataNo == 3)
                        g.DrawLine(new Pen(Color.Blue, 2), x1, y1, x2, y2);
                    else if (dataNo == 4)
                        g.DrawLine(new Pen(Color.DarkRed, 2), x1, y1, x2, y2);

                    x1 = x2; y1 = y2;
                }
            }
            //凡例
            g.DrawLine(new Pen(Color.Black, 2), 60, 15, 100, 15);
            g.DrawString("No=0", new Font("Century", 10), new SolidBrush(Color.Black), 100, 5);
            g.DrawLine(new Pen(Color.Red, 2), 160, 15, 200, 15);
            g.DrawString("No=1", new Font("Century", 10), new SolidBrush(Color.Black), 200, 5);
            g.DrawLine(new Pen(Color.Blue, 2), 260, 15, 300, 15);
            g.DrawString("No=2", new Font("Century", 10), new SolidBrush(Color.Black), 300, 5);
            g.DrawLine(new Pen(Color.DarkRed, 2), 360, 15, 400, 15);
            g.DrawString("No=3", new Font("Century", 10), new SolidBrush(Color.Black), 400, 5);

            g.Dispose();
            picSpace.Invalidate();
        }
        private void timer1_Tick(object sender, EventArgs e)
        {
            int i;
            float x0, y0, x1, y1;
            float scale = Convert.ToSingle(txtScale.Text);
            float scaleM = 0.7f * picSpace.Height / scale;//主目盛間隔
            if (txtDiameter.Text == "") return;
            float dia = Convert.ToSingle(txtDiameter.Text);
            float d = dia * scaleM;
            //左端位置
            x0 = 50.0f;
            y0 = (float)(picSpace.Height * 0.9f);

            g = Graphics.FromImage(picSpace.Image);

            //描画
            if (drawCount <= numMax)
            {
                i = drawCount;
                x1 = x0 + posY[i] * scaleM;
                y1 = y0 - (float)(posZ[i] * scaleM);
                if (dataNo == 1)
                    g.DrawEllipse(new Pen(Color.Black, 2), x1 - d / 2.0f, y1 - d / 2.0f, d, d);
                else if (dataNo == 2)
                    g.DrawEllipse(new Pen(Color.Red, 2), x1 - d / 2.0f, y1 - d / 2.0f, d, d);
                else if (dataNo == 3)
                    g.DrawEllipse(new Pen(Color.DarkBlue, 2), x1 - d / 2.0f, y1 - d / 2.0f, d, d);
                else if (dataNo == 4)
                    g.DrawEllipse(new Pen(Color.DarkRed, 2), x1 - d / 2.0f, y1 - d / 2.0f, d, d);

            }
            drawCount++;
            g.Dispose();
            picSpace.Invalidate();
        }

        private void txtScale_TextChanged(object sender, EventArgs e)
        {
            drawScale();
            listBox1.Items.Clear();
        }
    }
}