﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace CS_Collision2D
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }
        //-------------------------------------------------------------
        static int NUM_MAX = 1001;//データ個数最大値

        int numMax;        //実際のデータ個数
        double restitut;   //反発係数
        double mass1, mass2;
        double dia1, dia2;
        double scale;      //目盛間の長さ
        bool flagCollided = false;
        int drawCount;
        float[] posX1 = new float[NUM_MAX];
        float[] posY1 = new float[NUM_MAX];
        float[] posX2 = new float[NUM_MAX];
        float[] posY2 = new float[NUM_MAX];
        Graphics g;

        private void Form1_Load(object sender, EventArgs e)
        {
            txtSpeed1.Text = "30.0";//[m/s]
            txtAngle1.Text = "30.0";//[deg]
            txtMass1.Text = "1.0";  //[kg]
            txtDia1.Text = "1.0";//[m]
            txtSpeed2.Text = "30.0";//[m/s]
            txtAngle2.Text = "-30.0";//[deg]
            txtMass2.Text = "1.0";  //[kg]
            txtDia2.Text = "1.0";//[m]
            txtRestitut.Text = "0.2";
            txtScale.Text = "50.0";    //[m]
            txtTimeStep.Text = "0.001";//[s]
            txtInterval.Text = "0.1";  //[s]

            timer1.Enabled = false;
            drawScale();
        }

        private void drawScale()
        {
            float x0, y0;
            picSpace.Image = new Bitmap(picSpace.Width, picSpace.Height);
            g = Graphics.FromImage(picSpace.Image);
            g.Clear(Color.White);

            scale = Convert.ToDouble(txtScale.Text);
            if (scale == 0.0) return;
            double scaleM = 0.7 * picSpace.Height / scale;//主目盛間隔

            //中心位置
            x0 = (float)(picSpace.Width * 0.5f);
            y0 = (float)(picSpace.Height * 0.5f);
//MessageBox.Show(" x0=" + x0.ToString() + "   y0=" + y0.ToString());

            g = Graphics.FromImage(picSpace.Image);
            Pen pen = new Pen(Color.Blue, 1);
            //横軸
            g.DrawLine(pen, picSpace.Width * 0.05f, y0, picSpace.Width * 0.95f, y0);
            //横軸目盛
            g.DrawLine(pen, x0 - (float)(scale * scaleM), y0 + 10, x0 - (float)(scale * scaleM), y0 - 10);
            g.DrawLine(pen, x0, y0 + 10, x0, y0 - 10);
            g.DrawLine(pen, x0 + (float)(scale * scaleM), y0+10, x0 + (float)(scale * scaleM), y0 - 10);
            g.DrawString("-"+Convert.ToString(scale), new Font("Century", 10), new SolidBrush(Color.Blue), x0 - (float)(scale * scaleM) - 15, y0 + 7);
            g.DrawString("0", new Font("Century", 10), new SolidBrush(Color.Blue), x0-5, y0 + 7);
            g.DrawString(Convert.ToString(scale), new Font("Century", 10), new SolidBrush(Color.Blue), x0 + (float)(scale * scaleM) - 15, y0 + 7);

            g.Dispose();
            picSpace.Invalidate();
        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            double speed1 = Convert.ToDouble(txtSpeed1.Text);
            double angle1 = Convert.ToDouble(txtAngle1.Text);
            mass1 = Convert.ToDouble(txtMass1.Text);
            dia1 = Convert.ToDouble(txtDia1.Text);
            double speed2 = Convert.ToDouble(txtSpeed2.Text);
            double angle2 = Convert.ToDouble(txtAngle2.Text);
            mass2 = Convert.ToDouble(txtMass2.Text);
            dia2 = Convert.ToDouble(txtDia2.Text);
            double timeStep = Convert.ToDouble(txtTimeStep.Text);
            double timeInterval = Convert.ToDouble(txtInterval.Text);
            restitut = Convert.ToDouble(txtRestitut.Text); //反発係数

            CVector vPos1 = new CVector();
            CVector vPos2 = new CVector();
            CVector vVel1 = new CVector();
            CVector vVel2 = new CVector();

            //初期値
            vPos1.x = -scale;
            vPos1.y =  scale * Math.Tan(Math.PI * angle1 / 180.0);
            vPos2.x = -scale;
            vPos2.y =  scale * Math.Tan(Math.PI * angle2 / 180.0);
            vVel1.x =  speed1 * Math.Cos(Math.PI * angle1 / 180.0);
            vVel1.y = -speed1 * Math.Sin(Math.PI * angle1 / 180.0);
            vVel2.x =  speed2 * Math.Cos(Math.PI * angle2 / 180.0);
            vVel2.y = -speed2 * Math.Sin(Math.PI * angle2 / 180.0);
            posX1[0] = (float)vPos1.x;
            posY1[0] = (float)vPos1.y;
            posX2[0] = (float)vPos2.x;
            posY2[0] = (float)vPos2.y;

            drawCount = 0;
            timer1.Enabled = false;
            int numCount = 0;
            double timeCount = 0.0;
            flagCollided = false;
            while (numCount < NUM_MAX)
            {
                //オイラー法
                vPos1 += vVel1 * timeStep;
                vPos2 += vVel2 * timeStep;
                //衝突処理
                if(!flagCollided) collision(vPos1, vPos2, ref vVel1, ref vVel2);
               
                timeCount += timeStep;
                if (timeCount >= timeInterval)
                {
                    numCount++;
                    if (numCount >= NUM_MAX) break;
                    posX1[numCount] = (float)vPos1.x;
                    posY1[numCount] = (float)vPos1.y;
                    posX2[numCount] = (float)vPos2.x;
                    posY2[numCount] = (float)vPos2.y;
                    timeCount = 0.0;
                    if (vPos1.x > scale && vPos2.x > scale)
                    {
                        numMax = numCount;
                        break;
                    }
                }
            }

            drawCount = 0;
            timer1.Interval =  (int)(1000.0 * timeInterval);//ms
            timer1.Enabled = true;

        }
        //--------------------------------------------------------------
        private void collision(CVector vPos1, CVector vPos2, ref CVector vVel1, ref CVector vVel2)
        {
            CVector vNormal = new CVector();
            double rikiseki;

            double d = CVector.vDist(vPos1, vPos2);
            if (d <= (dia1 + dia2) / 2.0)
            {//衝突
                vNormal = vPos2 - vPos1;//法線方向
                vNormal.norm();
                rikiseki = (restitut + 1.0) * CVector.vDot(vNormal, (vVel2 - vVel1)) /(1.0 / mass1 + 1.0 / mass2);
                vVel1 += vNormal * rikiseki / mass1;
                vVel2 -= vNormal * rikiseki / mass2;
                flagCollided = true;
            }
        }
        //--------------------------------------------------------------
        private void drawGraph(float[] posX1, float[] posY1, float[] posX2, float[] posY2, int numMax)
        {
            int i;
            float x0, y0, x1, y1, x2, y2;
            float scaleM = 0.7f * picSpace.Height / (float)scale;//主目盛間隔
            float dia1 = Convert.ToSingle(txtDia1.Text);
            float dia2 = Convert.ToSingle(txtDia2.Text);
            float d1 = dia1 * scaleM;
            float d2 = dia2 * scaleM;

            //中心位置
            x0 = (float)(picSpace.Width * 0.5f);
            y0 = (float)(picSpace.Height * 0.5f);
            g = Graphics.FromImage(picSpace.Image);

            //描画
            for (i = 0; i <= numMax; i++)
            {
                x1 = x0 + posX1[i] * scaleM;
                y1 = y0 - (float)(posY1[i] * scaleM);
                g.DrawEllipse(new Pen(Color.Black, 2), (x1-d1/2.0f), (y1-d1/2.0f), d1, d1);

                x2 = x0 + posX2[i] * scaleM;
                y2 = y0 - (float)(posY2[i] * scaleM);
                g.DrawEllipse(new Pen(Color.Red, 2), x2-d2/2.0f, y2-d2/2.0f, d2, d2);
                g.Dispose();
                picSpace.Invalidate();
            }
            g.Dispose();
            picSpace.Invalidate();
        }

        private void txtScale_TextChanged(object sender, EventArgs e)
        {
            drawScale();
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            timer1.Enabled = false;
            g = Graphics.FromImage(picSpace.Image);
            g.Clear(Color.White);
            drawScale();
            g.Dispose();
            picSpace.Invalidate();
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            float x0, y0, x1, y1, x2, y2;
            float scaleM = 0.7f * picSpace.Height / (float)scale;//主目盛間隔
            if (txtDia1.Text == "") return;
            float dia1 = Convert.ToSingle(txtDia1.Text);
            if (txtDia2.Text == "") return;
            float dia2 = Convert.ToSingle(txtDia2.Text);
            float d1 = dia1 * scaleM;
            float d2 = dia2 * scaleM;
            int i;

            //座標原点
            x0 = (float)(picSpace.Width * 0.5f);
            y0 = (float)(picSpace.Height * 0.5f);
            g = Graphics.FromImage(picSpace.Image);

            //描画
            if(drawCount <= numMax)
            {
                i = drawCount;
                x1 = x0 + posX1[i] * scaleM;
                y1 = y0 - (float)(posY1[i] * scaleM);
                g.DrawEllipse(new Pen(Color.Black, 2), (x1 - d1 / 2.0f), (y1 - d1 / 2.0f), d1, d1);

                x2 = x0 + posX2[i] * scaleM;
                y2 = y0 - (float)(posY2[i] * scaleM);
                g.DrawEllipse(new Pen(Color.Red, 2), x2 - d2 / 2.0f, y2 - d2 / 2.0f, d2, d2);
                drawCount++;
            }
            g.Dispose();
            picSpace.Invalidate();
        }
    }
}