﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace CS_DoublePendulum
{
  public partial class Form1 : Form
  {
    public Form1()
    {
      InitializeComponent();
    }

//--------------------------------------------------------------------------
    const int NUM_MAX = 500;
    double actTime;//実行時間[s]
    int numData;
    string method;

    private void Form1_Load(object sender, EventArgs e)
    {
      cmbTimeStep.Items.Add("0.0001");
      cmbTimeStep.Items.Add("0.001");
      cmbTimeStep.Items.Add("0.002");
      cmbTimeStep.Items.Add("0.005");
      cmbTimeStep.Items.Add("0.01");
      cmbTimeStep.Text = "0.001";

      cmbActTime.Items.Add("100.0");
      cmbActTime.Items.Add("50.0");
      cmbActTime.Items.Add("25.0");
      cmbActTime.Text = ("100.0");

      cmbNumData.Items.Add("500");
      cmbNumData.Text = "500";

      listMethod.Items.Add("Euler");
      listMethod.Items.Add("Runge-Kutta");
      listMethod.Items.Add("Runge-Kutta-Gill");
      listMethod.Text = "Euler";

      txtTime0.Text = "0.0";  //グラフ表示の時間始点
      txtMass1.Text = "1.0";  //おもりの質量
      txtLength1.Text = "0.8";//糸の長さ
      txtTheta1.Text = "80.0";//振れ角
      txtOmega1.Text = "0.0"; //thetaの初角速度
      txtMass2.Text = "1.0";  //おもりの質量
      txtLength2.Text = "0.5";//糸の長さ
      txtTheta2.Text = "90.0";//振れ角
      txtOmega2.Text = "-10.0"; //初角速度
      txtViscous.Text = "0.0";//粘性抵抗係数
      txtAmplitude.Text = "2.0";
      picSpace1.Width = NUM_MAX + 100;
      picSpace2.Width = NUM_MAX + 100;
      picSpace3.Width = NUM_MAX + 100;
      picSpace1.Height = 170;
      picSpace2.Height = 170;
      picSpace3.Height = 170;

    }

    private void btnStart_Click(object sender, EventArgs e)
    {
      double timeStep = Convert.ToDouble(cmbTimeStep.Text);
      actTime = Convert.ToDouble(cmbActTime.Text);
      double mass1 = Convert.ToDouble(txtMass1.Text);
      double length1 = Convert.ToDouble(txtLength1.Text);
      double theta10 = Convert.ToDouble(txtTheta1.Text);
      double omega10 = Convert.ToDouble(txtOmega1.Text);
      double mass2 = Convert.ToDouble(txtMass2.Text);
      double length2 = Convert.ToDouble(txtLength2.Text);
      double theta20 = Convert.ToDouble(txtTheta2.Text);
      double omega20 = Convert.ToDouble(txtOmega2.Text);
      double viscous = Convert.ToDouble(txtViscous.Text);
      double time0 = Convert.ToDouble(txtTime0.Text);
      method = listMethod.Text;
      numData = Convert.ToInt32(cmbNumData.Text);
      //データ取り込み間隔
      int interval = (int)((actTime / timeStep) / NUM_MAX) / (NUM_MAX / numData);

      double alpha1, theta1, omega1;
      double alpha2, theta2, omega2;
      double g = 9.8;

      float[] yy2 = new float[NUM_MAX + 1];//おもり2のy座標
      float[] z2 = new float[NUM_MAX + 1];//おもり2のｚ座標（おもり1の自然の位置を0）

      //初期値
      theta1 = theta10 * Math.PI / 180.0;
      omega1 = omega10;
      theta2 = theta20 * Math.PI / 180.0;
      omega2 = omega20;

      double time = 0.0;
      double ss, cc;
      //double r2 = Math.Sqrt(2.0);
      int count = 0;
      int cnt = 0;
      double root2 = Math.Sqrt(2.0);

	    double aa, pa, qa, s1, s2;
      double len12, len21, mu, gl1, gl2;
      double k11, k12, k13, k14;
      double k21, k22, k23, k24;
      double r1, r2, h1, h2;
      //Runge-Kutta-Gillで使用する配列
      double[] kk1 = new double[5], kk2 = new double[5];
      double[] rr1 = new double[5], qq1 = new double[5];
      double[] uu = new double[5], rr2 = new double[5];
      double[] qq2 = new double[5], vv = new double[5];
      len12 = length1 / length2;
      len21 = length2 / length1;
      gl1 = g / length1;
      gl2 = g / length2;
      mu = mass2 / (mass1 + mass2);

      //Runge-Kutta-Gill法の初期値および係数
      qq1[0] = 0.0; qq2[0] = 0.0; uu[0] = omega10; vv[0] = omega20;
      double[] a = { 0.0, 0.5, 0.5 * (2.0 - root2), 0.5 * (2.0 + root2), 1.0 / 6.0 };
      double[] b = { 0.0, 2.0, 1.0, 1.0, 2.0 };
      double[] c = { 0.0, 0.5, 0.5 * (2.0 - root2), 0.5 * (2.0 + root2), 0.5 };

      while (time <= time0 + actTime && count < NUM_MAX)
      {
        ss = Math.Sin(theta1 - theta2);
        cc = Math.Cos(theta1 - theta2);
        s1 = Math.Sin(theta1);
        s2 = Math.Sin(theta2);
        aa = mu * cc * cc - 1.0;
        pa = mu * ss / aa;
        qa = gl1 / aa;
        if(method == "Euler")
        {
          alpha1 = pa * (omega1 * omega1 * cc + len21 * omega2 * omega2) + qa * (s1 - mu * cc * s2)
                   - (viscous / mass1) * omega1;
          alpha2 = len12 * (ss * omega1 * omega1 - cc * alpha1) - gl2 * s2
                   - (viscous / mass2) * omega2;
          omega1 += alpha1 * timeStep;
          omega2 += alpha2 * timeStep;
        }
        else if(method == "Runge-Kutta")
        {
          alpha1 = pa * (omega1 * omega1 * cc + len21 * omega2 * omega2) + qa * (s1 - mu * cc * s2)
                   - (viscous / mass1) * omega1;
          k11 = alpha1 * timeStep;
          k21 = (len12 * (ss * omega1 * omega1 - cc * alpha1) - gl2 * s2 - (viscous / mass2) * omega2) * timeStep;
          k12 = (pa * ((omega1 + k11 / 2.0) * (omega1 + k11 / 2.0) * cc + len21 * (omega2 + k21 / 2.0) * (omega2 + k21 / 2.0)) + qa * (s1 - mu * cc * s2)
                   - (viscous / mass1) * (omega1 + k11 / 2.0)) * timeStep;
          k22 = (len12 * (ss * (omega1 + k11 / 2.0) * (omega1 + k11 / 2.0) - cc * alpha1) - gl2 * s2
                   - (viscous / mass2) * (omega2 + k21 / 2.0)) * timeStep;
          k13 = (pa * ((omega1 + k12 / 2.0) * (omega1 + k12 / 2.0) * cc + len21 * (omega2 + k22 / 2.0) * (omega2 + k22 / 2.0)) + qa * (s1 - mu * cc * s2)
                   - (viscous / mass1) * (omega1 + k12 / 2.0)) * timeStep;
          k23 = (len12 * (ss * (omega1 + k12 / 2.0) * (omega1 + k12 / 2.0) - cc * alpha1) - gl2 * s2
                   - (viscous / mass2) * (omega2 + k22 / 2.0)) * timeStep;
          k14 = (pa * ((omega1 + k13) * (omega1 + k13) * cc + len21 * (omega2 + k23) * (omega2 + k23)) + qa * (s1 - mu * cc * s2)
                   - (viscous / mass1) * (omega1 + k13)) * timeStep;
          k24 = (len12 * (ss * (omega1 + k13) * (omega1 + k13) - cc * alpha1) - gl2 * s2
                   - (viscous / mass2) * (omega2 + k23)) * timeStep;

          omega1 += (k11 + 2.0 * k12 + 2.0 * k13 + k14) / 6.0;
          omega2 += (k21 + 2.0 * k22 + 2.0 * k23 + k24) / 6.0;
        }
        else//Runge-Kutta-Gill
        {
          alpha1 = pa * (omega1 * omega1 * cc + len21 * omega2 * omega2) + qa * (s1 - mu * cc * s2)
                   - (viscous / mass1) * omega1;

          for (int i = 1; i <= 4; i++)
          {
            kk1[i] = (pa * (uu[i - 1] * uu[i - 1] * cc + len21 * vv[i - 1] * vv[i - 1]) + qa * (s1 - mu * cc * s2) - (viscous / mass1) * uu[i - 1]) * timeStep;
            rr1[i] = a[i] * (kk1[i] - b[i] * qq1[i - 1]);
            uu[i] = uu[i - 1] + rr1[i];
            qq1[i] = qq1[i - 1] + 3.0 * rr1[i] - c[i] * kk1[i];
            kk2[i] = (len12 * (ss * uu[i - 1] * uu[i - 1] - cc * alpha1) - gl2 * s2 - (viscous / mass2) * vv[i - 1]) * timeStep;
            rr2[i] = a[i] * (kk2[i] - b[i] * qq2[i - 1]);
            vv[i] = vv[i - 1] + rr2[i];
            qq2[i] = qq2[i - 1] + 3.0 * rr2[i] - c[i] * kk2[i];
          }
          qq1[0] = qq1[4]; qq2[0] = qq2[4];
          uu[0] = uu[4]; vv[0] = vv[4];
          omega1 = uu[4];
          omega2 = vv[4];
        }
        theta1 += omega1 * timeStep;
        theta2 += omega2 * timeStep;
        r1 = length1 * s1;     //おもり1の水平方向座標
        r2 = r1 + length2 * s2;//おもり2の水平方向座標
        h1 = - length1 * Math.Cos(theta1);//おもり1のｚ座標(固定点を基準)
        h2 = h1 - length2 * Math.Cos(theta2);//おもり2のｚ座標

        if (time >= time0 && cnt == 0)
        {//interval回に1回保存
          yy2[count] = (float)r2;
          z2[count] = (float)(h2 + length1);
          count++;
        }
        cnt++;
        if (cnt == interval) cnt = 0;

        time += timeStep;
      }
      MessageBox.Show("time = " + time.ToString() + ",  count = " + count.ToString() + ",  interval= " + interval.ToString());

      drawGraph(yy2, z2);

    }
    private void drawGraph(float[] yy2, float[] z2)
    {
      int i;
      float x0, y0, x1, y1, x2, y2;

      double amp = Convert.ToDouble(txtAmplitude.Text);
      double scale = 0.4 * picSpace1.Height / amp;//angleの倍率
      //MessageBox.Show("scale = " + scale.ToString() + ",  scaleP = " + scaleP.ToString());
      //MessageBox.Show("angle[50] = " + angle[50].ToString() + ",  angle[100] = " + angle[100].ToString());

      //左端位置
      x0 = 50.0f;
      y0 = (float)(picSpace1.Height / 2.0f);

      Graphics g;
      if (method == "Euler")
      {
        picSpace1.Image = new Bitmap(picSpace1.Width, picSpace1.Height);
        g = Graphics.FromImage(picSpace1.Image);
      }
      else if (method == "Runge-Kutta")
      {
        picSpace2.Image = new Bitmap(picSpace2.Width, picSpace2.Height);
        g = Graphics.FromImage(picSpace2.Image);
      }
      else//Runge-Kutta-Gill
      {
        picSpace3.Image = new Bitmap(picSpace3.Width, picSpace3.Height);
        g = Graphics.FromImage(picSpace3.Image);
      }
      g.Clear(Color.White);

      Pen pen = new Pen(Color.Blue, 1);
      //縦軸
      g.DrawLine(pen, x0, y0 * 1.8f, x0, y0 * 0.2f);
      //横軸
      g.DrawLine(pen, x0, y0, x0 + (float)NUM_MAX, y0);
      //縦軸目盛
      g.DrawLine(pen, x0, y0 - (float)(amp * scale), x0 - 5.0f, y0 - (float)(amp * scale));
      g.DrawString(Convert.ToString(amp), new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 30, y0 - (float)(amp * scale + 10.0));
      g.DrawString("0", new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 20, y0 - 10);
      g.DrawString("-" + Convert.ToString(amp), new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 35, y0 + (float)(amp * scale - 10.0));
      g.DrawLine(pen, x0, y0 + (float)(amp * scale), x0 - 5.0f, y0 + (float)(amp * scale));
      //横軸目盛
      g.DrawLine(pen, x0 + 500, y0, x0 + 500, y0 - 20);
      g.DrawLine(pen, x0 + 250, y0, x0 + 250, y0 - 20);

      if (numData != 500) MessageBox.Show("numData=500としてください");
      if (actTime == 100.0) g.DrawString("100s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 480, y0 + 5);
      else if (actTime == 50.0) g.DrawString("50s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 480, y0 + 5);
      else if (actTime == 25.0) g.DrawString("25s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 480, y0 + 5);

      //yy2描画
      pen = new Pen(Color.Red, 2);
      x1 = x0;
      y1 = y0 - (float)(yy2[0] * scale);
      for (i = 1; i < NUM_MAX; i++)
      {
        x2 = x0 + i;
        y2 = y0 - (float)(yy2[i] * scale);
        g.DrawLine(pen, x1, y1, x2, y2);
        x1 = x2; y1 = y2;
      }
      //凡例
      g.DrawLine(pen, 200, 15, 250, 15);
      g.DrawString("y2[m]", new Font("Century", 10), new SolidBrush(Color.Red), 250, 5);
      //z2描画
      pen = new Pen(Color.Black, 2);
      x1 = x0;
      y1 = y0 - (float)(z2[0] * scale);
      for (i = 1; i < NUM_MAX; i++)
      {
        x2 = x0 + i;
        y2 = y0 - (float)(z2[i] * scale);
        g.DrawLine(pen, x1, y1, x2, y2);
        x1 = x2; y1 = y2;
      }
      //凡例
      g.DrawLine(pen, 60, 15, 110, 15);
      g.DrawString("z2[m]", new Font("Century", 10), new SolidBrush(Color.Black), 110, 5);
      if (method == "Euler") g.DrawString("Euler", new Font("Century", 10), new SolidBrush(Color.Blue), 350, 5);
      else if (method == "Runge-Kutta") g.DrawString("Runge-Kutta", new Font("Century", 10), new SolidBrush(Color.Blue), 350, 5);
      else if (method == "Runge-Kutta-Gill") g.DrawString("Runge-Kutta-Gill", new Font("Century", 10), new SolidBrush(Color.Blue), 350, 5);

      g.Dispose();
      picSpace1.Invalidate();
      picSpace2.Invalidate();
      picSpace3.Invalidate();

    }    
  }
}