﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
//追加した名前空間
using System.IO;//ファイル操作に必要
using System.Drawing.Imaging;//ImageFormatに必要

namespace CS_InversePendulum
{
  public partial class Form1 : Form
  {
    public Form1()
    {
      InitializeComponent();
    }
//-----------------------------------------------------------------
    const int NUM_MAX = 500;
    double actTime;//実行時間[s]
    int numData;
    string method;
    
    private void Form1_Load(object sender, EventArgs e)
    {
      cmbTimeStep.Items.Add("0.001");
      cmbTimeStep.Items.Add("0.005");
      cmbTimeStep.Items.Add("0.01");
      cmbTimeStep.Items.Add("0.05");
      cmbTimeStep.Text = "0.01";

      cmbActTime.Items.Add("100.0");
      cmbActTime.Items.Add("50.0");
      cmbActTime.Items.Add("25.0");
      cmbActTime.Text = ("100.0");

      cmbNumData.Items.Add("500");
      cmbNumData.Text = "500";

      listMethod.Items.Add("Euler");
      listMethod.Items.Add("Runge-Kutta-Gill");
      listMethod.Text = "Euler";

      txtMass0.Text = "0.1";   //おもりの質量
      txtMass1.Text = "1.0";  //台車の質量
      txtLength.Text = "1.0";
      txtTheta.Text = "30.0";
      txtViscous.Text = "0.0"; //粘性抵抗係数
      txtMuK.Text = "0.0";     //動摩擦抵抗
      txtK1.Text = "250.0";    //thetaに対する系数
      txtK2.Text = "10.0";     //omegaに対する系数
      txtK3.Text = "15.0";      //posCar
      txtK4.Text = "5.0";      //velCar
      txtAmplitude.Text = "40.0";
      picSpace.Width = NUM_MAX + 100;
    }

    private void btnStart_Click(object sender, EventArgs e)
    {
      double timeStep = Convert.ToDouble(cmbTimeStep.Text);
      actTime = Convert.ToDouble(cmbActTime.Text);
      double mass = Convert.ToDouble(txtMass0.Text);
      double MASS = Convert.ToDouble(txtMass1.Text);
      double length0 = Convert.ToDouble(txtLength.Text);
      double theta0 = Convert.ToDouble(txtTheta.Text);
      double viscous = Convert.ToDouble(txtViscous.Text);
      double muK = Convert.ToDouble(txtMuK.Text);
      double k1 = Convert.ToDouble(txtK1.Text);
      double k2 = Convert.ToDouble(txtK2.Text);
      double k3 = Convert.ToDouble(txtK3.Text);
      double k4 = Convert.ToDouble(txtK4.Text);
      method = listMethod.Text;

      numData = Convert.ToInt32(cmbNumData.Text);
      //データ取り込み間隔
      int interval = (int)((actTime / timeStep) / NUM_MAX) / (NUM_MAX / numData);

      double alpha, accelCar;
      double g = 9.8;


      double theta, omega, posCar, velCar;
      double[] angle = new double[NUM_MAX + 1];//thetaの確定値
      double[] pos = new double[NUM_MAX + 1];  //posCarの確定値
      double[] kt = new double[5];//theta
      double[] kc = new double[5];//cart
      double[] rt = new double[5];
      double[] qt = new double[5];
      double[] uu = new double[5];//omega
      double[] rc = new double[5];
      double[] qc = new double[5];
      double[] vv = new double[5];//velCar
      double root2 = Math.Sqrt(2.0);

      //初期値
      theta = theta0 * Math.PI / 180.0 ;
      omega = 0.0;
      posCar = 0.0;
      velCar = 0.0;

      double time = 0.0;
      double ss, cc, mm, force;
      int count = 0;
      int cnt = 0;

      //Runge-Kutta-Gill法の初期値および係数
      qt[0] = 0.0; qc[0] = 0.0; uu[0] = 0.0; vv[0] = 0.0;
      double[] a = { 0.0, 0.5, 0.5 * (2.0 - root2), 0.5 * (2.0 + root2), 1.0 / 6.0 };
      double[] b = { 0.0, 2.0, 1.0, 1.0, 2.0 };
      double[] c = { 0.0, 0.5, 0.5 * (2.0 - root2), 0.5 * (2.0 + root2), 0.5 };

      while (time <= actTime && count < NUM_MAX)
      {
        ss = Math.Sin(theta);
        cc = Math.Cos(theta);
        force = k1 * ss + k2 * omega * cc + k3 * posCar + k4 * velCar;
        mm = MASS + mass - mass * cc * cc;
        if (method == "Euler")
        {
          //オイラー法
          //おもり
          alpha = ((MASS + mass) * g * ss - force * cc - mass * length0 * ss * cc * omega) / (length0 * mm) - viscous * omega / mass;
          omega += alpha * timeStep;
          theta += omega * timeStep;//rad
          //台車
          if (velCar > 0.0)
            accelCar = (force + mass * length0 * ss * omega * omega - mass * g * ss * cc) / mm - muK * g;
          else
            accelCar = (force + mass * length0 * ss * omega * omega - mass * g * ss * cc) / mm + muK * g;
          velCar += accelCar * timeStep;
          posCar += velCar * timeStep;
          
        }
        else//Runge-Kutta-Gill
        {
          for (int i = 1; i <= 4; i++)
          {
            kt[i] = (((MASS + mass) * g * ss - force * cc - mass * length0 * ss * cc * uu[i-1]) / (length0 * mm) - viscous * uu[i-1] / mass) * timeStep;
            rt[i] = a[i] * (kt[i] - b[i] * qt[i - 1]);
            uu[i] = uu[i - 1] + rt[i];
            qt[i] = qt[i - 1] + 3.0 * rt[i] - c[i] * kt[i];
            if (velCar > 0.0)
              kc[i] = ((force + mass * length0 * ss * uu[i - 1] * uu[i - 1] - mass * g * ss * cc) / mm - muK * g) * timeStep;
            else
              kc[i] = ((force + mass * length0 * ss * uu[i - 1] * uu[i - 1] - mass * g * ss * cc) / mm + muK * g) * timeStep;
            rc[i] = a[i] * (kc[i] - b[i] * qc[i - 1]);
            vv[i] = vv[i - 1] + rc[i];
            qc[i] = qc[i - 1] + 3.0 * rc[i] - c[i] * kc[i];
          }
          qt[0] = qt[4]; qc[0] = qc[4];
          uu[0] = uu[4]; vv[0] = vv[4];
          omega = uu[4];
          velCar = vv[4];
          theta += omega * timeStep;//rad
          posCar += velCar * timeStep;
        }

        if (cnt == 0)
        {//interval回に1回保存
          angle[count] = theta * 180.0 / Math.PI;
          pos[count] = posCar;
          count++;
        }
        cnt++;
        if (cnt == interval) cnt = 0;

        time += timeStep;
      }
 MessageBox.Show("time = " + time.ToString() + ",  count = " + count.ToString()+"  pos[5]="+pos[5].ToString());

      drawGraph(angle, pos);
    }

    private void drawGraph(double[] angle, double[] pos)
    {
      int i;
      float x0, y0, x1, y1, x2, y2;

      float amp = Convert.ToSingle(txtAmplitude.Text);
      float scaleA = (float)(0.4 * picSpace.Height / amp);//angleの倍率
      float scaleP = (float)(0.4 * picSpace.Height / amp);//posの倍率
 //MessageBox.Show("scaleA = " + scaleA.ToString() + ",  scaleP = " + scaleP.ToString());
 //MessageBox.Show("angle[400] = " + angle[400].ToString() + ",  angle[499] = " + angle[499].ToString());
 //MessageBox.Show("pos[400] = " + pos[400].ToString() + ",  pos[499] = " + pos[499].ToString());

      //左端位置
      x0 = 50.0f;
      y0 = (float)(picSpace.Height / 2.0f);

      picSpace.Image = new Bitmap(picSpace.Width, picSpace.Height);
      Graphics g = Graphics.FromImage(picSpace.Image);
      g.Clear(Color.White);

      Pen pen = new Pen(Color.Blue, 1);
      //縦軸
      g.DrawLine(pen, x0, y0 * 1.8f, x0, y0 * 0.2f);
      //横軸
      g.DrawLine(pen, x0, y0, x0 + (float)NUM_MAX, y0);
      //縦軸目盛
      g.DrawLine(pen, x0, y0 - (float)(amp * scaleA), x0 - 5.0f, y0 - (float)(amp * scaleA));
      g.DrawString(Convert.ToString(amp), new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 30, y0 - (float)(amp * scaleA + 10.0));
      g.DrawString("0", new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 20, y0 - 10);
      g.DrawString("-"+Convert.ToString(amp), new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 35, y0 + (float)(amp * scaleA - 10.0));
      g.DrawLine(pen, x0, y0 + (float)(amp * scaleA), x0 - 5.0f, y0 + (float)(amp * scaleA));
      //横軸目盛
      g.DrawLine(pen, x0 + 500, y0, x0 + 500, y0 - 20);
      g.DrawLine(pen, x0 + 250, y0, x0 + 250, y0 - 20);
      if (numData != 500) MessageBox.Show("numData=500としてください");
      if (actTime == 100.0) g.DrawString("100s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 480, y0 + 5);
      else if (actTime == 50.0) g.DrawString("50s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 480, y0 + 5);
      else if (actTime == 25.0) g.DrawString("25s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 480, y0 + 5);
      else if (numData == 125) g.DrawString("25s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 490, y0 + 5);
 
      //振れ角（theta, angle)描画
      pen = new Pen(Color.Black, 2);
      x1 = x0;
      y1 = y0 - (float)(angle[0] * scaleA);
      for (i = 1; i < NUM_MAX; i++)
      {
        x2 = x0 + i;
        y2 = y0 - (float)(angle[i] * scaleA);
//MessageBox.Show("y1 = " + y1.ToString() + ",  y2 = " + y2.ToString());
        g.DrawLine(pen, x1, y1, x2, y2);
        x1 = x2; y1 = y2;
      }
      //凡例
      g.DrawLine(pen, 100, 15, 150, 15);
      g.DrawString("θ[deg]", new Font("Century", 10), new SolidBrush(Color.Black), 150, 5);

      //位置（posCar, pos)描画
      pen = new Pen(Color.Red, 2);
      x1 = x0;
      y1 = y0 - (float)(pos[0] * scaleP * 10.0);

      for (i = 1; i < NUM_MAX; i++)
      {
        x2 = x0 + i;
        y2 = y0 - (float)(pos[i] * scaleP * 10.0);
        g.DrawLine(pen, x1, y1, x2, y2);
        x1 = x2; y1 = y2;
      }
      //凡例
      g.DrawLine(pen, 250, 15, 300, 15);
      g.DrawString("10*y[m]", new Font("Century", 10), new SolidBrush(Color.Black), 300, 5);
      if(method == "Euler")
        g.DrawString("Euler", new Font("Century", 10), new SolidBrush(Color.Blue), 400, 5);
      else
        g.DrawString("Runge-Kutta-Gill", new Font("Century", 10), new SolidBrush(Color.Blue), 400, 5);

      g.Dispose();
      picSpace.Invalidate();

    }
  }
}