﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace CS_SinglePendulum
{
  public partial class Form1 : Form
  {
    public Form1()
    {
      InitializeComponent();
    }

//-----------------------------------------------------------------
    const int NUM_MAX = 500;
    double actTime;//実行時間[s]
    int numData;
    string method;

    private void Form1_Load(object sender, EventArgs e)
    {
      cmbTimeStep.Items.Add("0.0001");
      cmbTimeStep.Items.Add("0.001");
      cmbTimeStep.Items.Add("0.002");
      cmbTimeStep.Items.Add("0.005");
      cmbTimeStep.Items.Add("0.01");
      cmbTimeStep.Text = "0.001";

      cmbActTime.Items.Add("1000.0");
      cmbActTime.Items.Add("500.0");
      cmbActTime.Items.Add("200.0");
      cmbActTime.Items.Add("100.0");
      cmbActTime.Text = "200.0";

      cmbNumData.Items.Add("500");
      cmbNumData.Text = "500";//固定

      listMethod.Items.Add("Euler");
      listMethod.Items.Add("Runge-Kutta");
      listMethod.Items.Add("Runge-Kutta-Gill");
      listMethod.Text = "Euler";

      txtMass.Text = "1.0";   //おもりの質量
      txtLength.Text = "1.0";//糸の長さ
      txtTheta.Text = "50.0"; //振れ角
      txtPhi.Text = "90.0"; //振れ角
      txtOmegaT.Text = "-1.0";//thetaの初角速度
      txtOmegaP.Text = "4.0"; //phiの初角速度
      txtViscous.Text = "0.0";//粘性抵抗係数
      txtAmplitude.Text = "100.0";
      picSpace1.Width = NUM_MAX + 100;
      picSpace2.Width = NUM_MAX + 100;
      picSpace3.Width = NUM_MAX + 100;
      picSpace1.Height = 180;
      picSpace2.Height = 180;
      picSpace3.Height = 180;
    }

    private void btnStart_Click(object sender, EventArgs e)
    {
      double timeStep = Convert.ToDouble(cmbTimeStep.Text);
      actTime = Convert.ToDouble(cmbActTime.Text);
      double mass = Convert.ToDouble(txtMass.Text);
      double length0 = Convert.ToDouble(txtLength.Text);
      double theta0 = Convert.ToDouble(txtTheta.Text);
      double phi0 = Convert.ToDouble(txtPhi.Text);
      double omegaT0 = Convert.ToDouble(txtOmegaT.Text);
      double omegaP0 = Convert.ToDouble(txtOmegaP.Text);
      double viscous = Convert.ToDouble(txtViscous.Text);
      method = listMethod.Text;
      numData = Convert.ToInt32(cmbNumData.Text);
      //データ取り込み間隔
      int interval = (int)((actTime / timeStep) / NUM_MAX) / (NUM_MAX / numData);

      double alphaT, alphaP, theta, omegaT, phi, omegaP;
      double g = 9.8;

      float[] angleT = new float[NUM_MAX + 1];//thetaの確定値

      //初期値
      theta = theta0 * Math.PI / 180.0;
      omegaT = omegaT0;
      phi = phi0 * Math.PI / 180.0;
      omegaP = omegaP0;

      double time = 0.0;
      double ss, cc;
      double[] kt = new double[5];
      double[] kp = new double[5];
      double[] rt = new double[5];
      double[] qt = new double[5];
      double[] uu = new double[5];
      double[] rp = new double[5];
      double[] qp = new double[5];
      double[] vv = new double[5];
      double root2 = Math.Sqrt(2.0);
      int count = 0;
      int cnt = 0;

      //Runge-Kutta-Gill法の初期値および係数
      qt[0] = 0.0; qp[0] = 0.0; uu[0] = omegaT0; vv[0] = omegaP0;
      double[] a = { 0.0, 0.5, 0.5 * (2.0 - root2), 0.5 * (2.0 + root2), 1.0 / 6.0 };
      double[] b = { 0.0, 2.0, 1.0, 1.0, 2.0 };
      double[] c = { 0.0, 0.5, 0.5 * (2.0 - root2), 0.5 * (2.0 + root2), 0.5 };
      while (time <= actTime && count < NUM_MAX)
      {
        ss = Math.Sin(theta);
        cc = Math.Cos(theta);
        
        if (Math.Abs(omegaP0) <= 0.00001)
        {//単振り子として解く(オイラー法だけ）
          alphaT = -(g / length0) * ss - (viscous / mass) * omegaT;
          alphaP = 0.0;
          omegaT += alphaT * timeStep;
          omegaP += alphaP * timeStep;
        }
        else
        {
          if(Math.Abs(ss) < 0.00001) break;
          if (Math.Abs(theta) > Math.PI / 2.0) break;
 
          if(method == "Euler")
          {
            alphaT = ss * cc * omegaP * omegaP - (g / length0) * ss - (viscous / mass) * omegaT;
            alphaP = -2.0 * (cc / ss) * omegaT * omegaP - (viscous / mass) * omegaP;
            omegaT += alphaT * timeStep;
            omegaP += alphaP * timeStep;
          }

          else if (method == "Runge-Kutta")
          {

            kt[0] = 0.0; kp[0] = 0.0;
            for (int i = 1; i <= 4; i++)
            {
                if (i == 1 || i == 4)
                {
                    kt[i] = (ss * cc * (omegaP + kp[i - 1]) * (omegaP + kp[i - 1]) - (g / length0) * ss - (viscous / mass) * (omegaT + kt[i - 1])) * timeStep;
                    kp[i] = (-2.0 * (cc / ss) * (omegaT + kt[i - 1]) * (omegaP + kp[i - 1]) - (viscous / mass) * (omegaP + kp[i - 1])) * timeStep;
                }
                else
                {
                    kt[i] = (ss * cc * (omegaP + kp[i - 1] / 2.0) * (omegaP + kp[i - 1] / 2.0) - (g / length0) * ss - (viscous / mass) * (omegaT + kt[i - 1] / 2.0)) * timeStep;
                    kp[i] = (-2.0 * (cc / ss) * (omegaT + kt[i - 1] / 2.0) * (omegaP + kp[i - 1] / 2.0) - (viscous / mass) * (omegaP + kp[i - 1] / 2.0)) * timeStep;
                }
            }
            omegaT += (kt[1] + 2.0 * kt[2] + 2.0 * kt[3] + kt[4]) / 6.0;//alphaT * timeStep;
            omegaP += (kp[1] + 2.0 * kp[2] + 2.0 * kp[3] + kp[4]) / 6.0;//alphaP * timeStep;
          }

          else//Runge-Kutta-Gill
          {
            for (int i = 1; i <= 4; i++)
            {
              kt[i] = (ss * cc * vv[i - 1] * vv[i - 1] - (g / length0) * ss - (viscous / mass) * uu[i - 1]) * timeStep;
              rt[i] = a[i] * (kt[i] - b[i] * qt[i - 1]);
              uu[i] = uu[i - 1] + rt[i];
              qt[i] = qt[i - 1] + 3.0 * rt[i] - c[i] * kt[i];
              kp[i] = (-2.0 * (cc / ss) * uu[i - 1] * vv[i - 1] - (viscous / mass) * vv[i - 1]) * timeStep;
              rp[i] = a[i] * (kp[i] - b[i] * qp[i-1]);
              vv[i] = vv[i-1] + rp[i];
              qp[i] = qp[i-1] + 3.0 * rp[i] - c[i] * kp[i];
            }
            qt[0] = qt[4]; qp[0] = qp[4];
            uu[0] = uu[4]; vv[0] = vv[4];
            omegaT = uu[4];
            omegaP = vv[4];
          }
        }
        theta += omegaT * timeStep;
        phi += omegaP * timeStep;
 
        if (cnt == 0)
        {//interval回に1回保存
          angleT[count] = (float)(theta * 180.0 / Math.PI);
          count++;
        }
        cnt++;
        if (cnt == interval) cnt = 0;

        time += timeStep;
      }
 MessageBox.Show("time = " + time.ToString() + ",  count = " + count.ToString()+ ",  interval= " + interval.ToString());

      drawGraph(angleT);

    }

    private void drawGraph(float[] angleT)
    {
      int i;
      float x0, y0, x1, y1, x2, y2;

      double amp = Convert.ToDouble(txtAmplitude.Text);
      double scaleA = 0.6 * picSpace1.Height / amp;//angleの倍率
      //MessageBox.Show("scaleA = " + scaleA.ToString() + ",  scaleP = " + scaleP.ToString());
      //MessageBox.Show("angle[50] = " + angle[50].ToString() + ",  angle[100] = " + angle[100].ToString());

      //左端位置
      x0 = 50.0f;
      y0 = (float)(picSpace1.Height * 0.8f);// / 2.0f);

      Graphics g;
      if (method == "Euler")
      {
        picSpace1.Image = new Bitmap(picSpace1.Width, picSpace1.Height);
        g = Graphics.FromImage(picSpace1.Image);
      }
      else if (method == "Runge-Kutta")
      {
        picSpace2.Image = new Bitmap(picSpace2.Width, picSpace2.Height);
        g = Graphics.FromImage(picSpace2.Image);
      }
      else//Runge-Kutta-Gill
      {
        picSpace3.Image = new Bitmap(picSpace3.Width, picSpace3.Height);
        g = Graphics.FromImage(picSpace3.Image);
      }
      g.Clear(Color.White);

      Pen pen = new Pen(Color.Blue, 1);
      //縦軸
      g.DrawLine(pen, x0, y0 * 1.1f, x0, y0 * 0.1f);
      //横軸
      g.DrawLine(pen, x0, y0, x0 + (float)NUM_MAX, y0);
      //縦軸目盛
      g.DrawLine(pen, x0, y0 - (float)(amp * scaleA), x0 - 5.0f, y0 - (float)(amp * scaleA));
      g.DrawString(Convert.ToString(amp), new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 40, y0 - (float)(amp * scaleA + 10.0));
      g.DrawString("0", new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 20, y0 - 10);
      //横軸目盛
      g.DrawLine(pen, x0 + 500, y0, x0 + 500, y0 - 20);
      g.DrawLine(pen, x0 + 250, y0, x0 + 250, y0 - 20);
      
      if (numData != 500) MessageBox.Show("numData=500としてください");
      if(actTime == 100.0)  g.DrawString("100s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 480, y0 + 5);
      else if (actTime == 200.0) g.DrawString("200s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 480, y0 + 5);
      else if (actTime == 500.0) g.DrawString("500s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 480, y0 + 5);
      else if (actTime == 1000.0) g.DrawString("1000s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 480, y0 + 5);

      //振れ角（theta, angleT)描画
      pen = new Pen(Color.Black, 2);
      x1 = x0;
      y1 = y0 - (float)(angleT[0] * scaleA);
      for (i = 1; i < NUM_MAX; i++)
      {
        x2 = x0 + i;
        y2 = y0 - (float)(angleT[i] * scaleA);
        g.DrawLine(pen, x1, y1, x2, y2);
        x1 = x2; y1 = y2;
      }
      //凡例
      g.DrawLine(pen, 100, 15, 150, 15);
      g.DrawString("θ[deg]", new Font("Century", 10), new SolidBrush(Color.Black), 150, 5);
      if(method == "Euler") g.DrawString("Euler", new Font("Century", 10), new SolidBrush(Color.Red), 350, 5);
      else if(method == "Runge-Kutta") g.DrawString("Runge-Kutta", new Font("Century", 10), new SolidBrush(Color.Red), 350, 5);
      else if (method == "Runge-Kutta-Gill") g.DrawString("Runge-Kutta-Gill", new Font("Century", 10), new SolidBrush(Color.Red), 350, 5);

      g.Dispose();
      picSpace1.Invalidate();

    }
  }
}