﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace CS_CoupledPendula0
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }
        //--------------------------------------------------------------------------
        const int NUM_MAX = 500;
        double actTime;//実行時間[s]
        int numData;

        private void Form1_Load(object sender, EventArgs e)
        {
            cmbTimeStep.Items.Add("0.00001");
            cmbTimeStep.Items.Add("0.0001");
            cmbTimeStep.Items.Add("0.001");
            cmbTimeStep.Items.Add("0.01");
            cmbTimeStep.Text = "0.0001";

            cmbActTime.Items.Add("200.0");
            cmbActTime.Items.Add("100.0");
            cmbActTime.Items.Add("50.0");
            cmbActTime.Items.Add("25.0");
            cmbActTime.Items.Add("10.0");
            cmbActTime.Items.Add("5.0");
            cmbActTime.Text = ("100.0");

            cmbNumData.Items.Add("500");
            cmbNumData.Text = "500";

            txtTime0.Text = "0.0"; //グラフ表示の時間始点
            txtMass1.Text = "1.0";  //おもりの質量
            txtMass2.Text = "1.0";  //おもりの質量
            txtLength1.Text = "1.0";//棒の長さ
            txtLength2.Text = "1.0";//棒の長さ
            txtLength0.Text = "1.0";//バネの自然長さ
            txtTheta1.Text = "30.0";//振れ角
            txtTheta2.Text = "0.0";//振れ角
            txtVelocity1X.Text = "1.0";
            txtConstK.Text = "1.0";//ばね定数
            txtViscous.Text = "0.0";//粘性抵抗係数
            txtAmplitude.Text = "1.0";

            picSpace1.Width = NUM_MAX + 80;
            picSpace2.Width = NUM_MAX + 80;
        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            double timeStep = Convert.ToDouble(cmbTimeStep.Text);
            actTime = Convert.ToDouble(cmbActTime.Text);
            double mass1 = Convert.ToDouble(txtMass1.Text);
            double mass2 = Convert.ToDouble(txtMass2.Text);
            double length1 = Convert.ToDouble(txtLength1.Text);
            double length2 = Convert.ToDouble(txtLength2.Text);
            double length0 = Convert.ToDouble(txtLength0.Text);
            double theta10 = Convert.ToDouble(txtTheta1.Text);
            double theta20 = Convert.ToDouble(txtTheta2.Text);
            double constK = Convert.ToDouble(txtConstK.Text);
            double viscous = Convert.ToDouble(txtViscous.Text);
            double velocity1X = Convert.ToDouble(txtVelocity1X.Text);
            double time0 = Convert.ToDouble(txtTime0.Text);
          
            numData = Convert.ToInt32(cmbNumData.Text);
            //データ取り込み間隔
            int interval = (int)((actTime / timeStep) / NUM_MAX) / (NUM_MAX / numData);

            double g = 9.8;
            CVector vGravity = new CVector(0.0, 0.0, -g);

            float[] xx1 = new float[NUM_MAX + 1];//おもりのy座標
            float[] zz1 = new float[NUM_MAX + 1];//おもりのｚ座標（おもり1の自然の位置を0）
            float[] xx2 = new float[NUM_MAX + 1];//おもりのy座標
            float[] zz2 = new float[NUM_MAX + 1];//おもりのｚ座標（おもり1の自然の位置を0）

            double cc1, cc2, x1, x2, h1, h2;
            double len, disp, omega1, omega2;
            CVector vAcc1, vAcc2, vTension1, vTension2 = new CVector();
            CVector vDir1, vDir2, vDir0, vVel1, vVel2 = new CVector();

            //初期値
            double heightFix = 2.0;//適当
            double theta1 = theta10 * Math.PI / 180.0;
            double theta2 = theta20 * Math.PI / 180.0;
            double theta01 = theta1;//前タイムステップの振れ角
            double theta02 = theta2;
            //初期y座標
            double y10 = length1 * Math.Sin(theta1);
            double y20 = length2 * Math.Sin(theta2);
            //初期のおもりの高さ
            double height1 = heightFix - length1 * Math.Cos(theta1);
            double height2 = heightFix - length2 * Math.Cos(theta2);
            //位置
            CVector vPos1 = new CVector(-length0 / 2.0 + y10, 0.0, height1);
            CVector vPos2 = new CVector( length0 / 2.0 + y20, 0.0, height2);
            //初速度
            vVel1 = new CVector(velocity1X, 0.0, 0.0); 
            vVel2 = new CVector(0.0, 0.0, 0.0); 
            //固定点
            CVector vPosFix1 = new CVector(-length0 / 2.0, 0.0, heightFix);
            CVector vPosFix2 = new CVector( length0 / 2.0, 0.0, heightFix);

            double time = 0.0;
            int count = 0;
            int cnt = 0;

            while (time <= time0 + actTime && count < NUM_MAX)
            {
                //Euler
			    x1 = CVector.vDist(new CVector(-length0/2.0, 0.0, vPos1.z), vPos1);
			    h1 = heightFix - vPos1.z;
			    x2 = CVector.vDist(new CVector( length0/2.0, 0.0, vPos2.z), vPos2); 		
			    h2 = heightFix - vPos2.z;
			    theta1 = Math.Atan2(x1, h1);
                theta2 = Math.Atan2(x2, h2);
			    omega1 = (theta1 - theta01) / timeStep;
			    omega2 = (theta2 - theta02) / timeStep;
                cc1 = Math.Cos(theta1);
                cc2 = Math.Cos(theta2);
			    //棒の単位ベクトル
			    vDir1 = vPosFix1 - vPos1;
                vDir1.norm();
			    vDir2 = vPosFix2 - vPos2;
                vDir2.norm();
			    //張力
			    vTension1 = (length1 * omega1 * omega1 + g * cc1) * vDir1;//mass1は除いてある
			    vTension2 = (length2 * omega2 * omega2 + g * cc2) * vDir2;//mass2は除いてある
    			
			    len = CVector.vDist(vPos1, vPos2);//バネの長さ
			    disp = len - length0; //変位
			    vDir0 = vPos2 - vPos1;//バネの単位ベクトル
                vDir0.norm();
			    vAcc1 = vTension1 + constK * disp *vDir0 / mass1 + vGravity - (viscous/mass1) * vVel1;
			    vAcc2 = vTension2 - constK * disp *vDir0 / mass2 + vGravity - (viscous/mass2) * vVel2;

			    vVel1 += vAcc1 * timeStep;
			    vVel2 += vAcc2 * timeStep;
			    vPos1 += vVel1 * timeStep;
			    vPos2 += vVel2 * timeStep;
			    //固定点が移動しないようにｚ座標を振れ角から求める
			    vPos1.z = heightFix - length1 * cc1;
			    vPos2.z = heightFix - length2 * cc2;
			    theta01 = theta1;//現在の振れ角を保存
			    theta02 = theta2;

                if (time >= time0 && cnt == 0)
                {//interval回に1回保存
                    xx1[count] = (float)(vPos1.x + length0 / 2.0);
                    zz1[count] = (float)(vPos1.z -(heightFix - length1));//鉛直軸自然長の位置を基準
                    xx2[count] = (float)(vPos2.x - length0 / 2.0);
                    zz2[count] = (float)(vPos2.z -(heightFix - length2));//鉛直軸自然長の位置を基準
                    count++;
                }
                cnt++;
                if (cnt == interval) cnt = 0;

                time += timeStep;
            }
            MessageBox.Show("time = " + time.ToString() + ",  count = " + count.ToString() + ",  interval= " + interval.ToString());

            drawGraph1(xx1, zz1);
            drawGraph2(xx2, zz2);
 
        }
        private void drawGraph1(float[] xx1, float[] zz1)
        {
            int i;
            float x0, y0, x1, y1, x2, y2;

            double amp = Convert.ToDouble(txtAmplitude.Text);
            double scale = 0.4 * picSpace1.Height / amp;//倍率

            //Lagrange
            //左端位置
            x0 = 50.0f;
            y0 = (float)(picSpace1.Height / 2.0f);

            Graphics g;
            picSpace1.Image = new Bitmap(picSpace1.Width, picSpace1.Height);
            g = Graphics.FromImage(picSpace1.Image);
            g.Clear(Color.White);

            Pen pen = new Pen(Color.Blue, 1);
            //縦軸
            g.DrawLine(pen, x0, y0 * 1.8f, x0, y0 * 0.2f);
            //横軸
            g.DrawLine(pen, x0, y0, x0 + (float)NUM_MAX, y0);
            //縦軸目盛
            g.DrawLine(pen, x0, y0 - (float)(amp * scale), x0 - 5.0f, y0 - (float)(amp * scale));
            g.DrawString(Convert.ToString(amp), new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 30, y0 - (float)(amp * scale + 10.0));
            g.DrawString("0", new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 20, y0 - 10);
            g.DrawString("-" + Convert.ToString(amp), new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 35, y0 + (float)(amp * scale - 10.0));
            g.DrawLine(pen, x0, y0 + (float)(amp * scale), x0 - 5.0f, y0 + (float)(amp * scale));
            //横軸目盛
            g.DrawLine(pen, x0 + 500, y0, x0 + 500, y0 - 20);
            g.DrawLine(pen, x0 + 250, y0, x0 + 250, y0 - 20);

            if (numData != 500) MessageBox.Show("numData=500としてください");
            if (actTime == 200.0) g.DrawString("200s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 480, y0 + 5);
            else if (actTime == 100.0) g.DrawString("100s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 480, y0 + 5);
            else if (actTime == 50.0) g.DrawString("50s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 480, y0 + 5);
            else if (actTime == 10.0) g.DrawString("10s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 480, y0 + 5);
            else if (actTime == 5.0) g.DrawString("5s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 490, y0 + 5);

            //xx描画
            pen = new Pen(Color.Red, 2);
            x1 = x0;
            y1 = y0 - (float)(xx1[0] * scale);
            for (i = 1; i < NUM_MAX; i++)
            {
                x2 = x0 + i;
                y2 = y0 - (float)(xx1[i] * scale);
                g.DrawLine(pen, x1, y1, x2, y2);
                x1 = x2; y1 = y2;
            }
            //凡例
            g.DrawLine(pen, 150, 15, 180, 15);
            g.DrawString("x[m]", new Font("Century", 10), new SolidBrush(Color.Red), 180, 5);
            //zz描画
            pen = new Pen(Color.Black, 2);
            x1 = x0;
            y1 = y0 - (float)(zz1[0] * scale);
            for (i = 1; i < NUM_MAX; i++)
            {
                x2 = x0 + i;
                y2 = y0 - (float)(zz1[i] * scale);
                g.DrawLine(pen, x1, y1, x2, y2);
                x1 = x2; y1 = y2;
            }
            //凡例
            g.DrawLine(pen, 60, 15, 90, 15);
            g.DrawString("z[m]", new Font("Century", 10), new SolidBrush(Color.Black), 90, 5);
            g.DrawString("weight1", new Font("Century", 10), new SolidBrush(Color.Blue), 350, 5);
            g.Dispose();
            picSpace1.Invalidate();
        }

        private void drawGraph2(float[] xx2, float[] zz2)
        {
            int i;
            float x0, y0, x1, y1, x2, y2;

            double amp = Convert.ToDouble(txtAmplitude.Text);
            double scale = 0.4 * picSpace1.Height / amp;//倍率
            //簡易解法
            //左端位置
            x0 = 50.0f;
            y0 = (float)(picSpace2.Height / 2.0f);

            Graphics g;
            picSpace2.Image = new Bitmap(picSpace2.Width, picSpace2.Height);
            g = Graphics.FromImage(picSpace2.Image);
            g.Clear(Color.White);

            Pen pen = new Pen(Color.Blue, 1);
            //縦軸
            g.DrawLine(pen, x0, y0 * 1.8f, x0, y0 * 0.2f);
            //横軸
            g.DrawLine(pen, x0, y0, x0 + (float)NUM_MAX, y0);
            //縦軸目盛
            g.DrawLine(pen, x0, y0 - (float)(amp * scale), x0 - 5.0f, y0 - (float)(amp * scale));
            g.DrawString(Convert.ToString(amp), new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 30, y0 - (float)(amp * scale + 10.0));
            g.DrawString("0", new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 20, y0 - 10);
            g.DrawString("-" + Convert.ToString(amp), new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 35, y0 + (float)(amp * scale - 10.0));
            g.DrawLine(pen, x0, y0 + (float)(amp * scale), x0 - 5.0f, y0 + (float)(amp * scale));
            //横軸目盛
            g.DrawLine(pen, x0 + 500, y0, x0 + 500, y0 - 20);
            g.DrawLine(pen, x0 + 250, y0, x0 + 250, y0 - 20);

            if (numData != 500) MessageBox.Show("numData=500としてください");
            if (actTime == 200.0) g.DrawString("200s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 480, y0 + 5);
            else if (actTime == 100.0) g.DrawString("100s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 480, y0 + 5);
            else if (actTime == 50.0) g.DrawString("50s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 480, y0 + 5);
            else if (actTime == 10.0) g.DrawString("10s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 480, y0 + 5);
            else if (actTime == 5.0) g.DrawString("5s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 490, y0 + 5);

            //xx描画
            pen = new Pen(Color.Red, 2);
            x1 = x0;
            y1 = y0 - (float)(xx2[0] * scale);
            for (i = 1; i < NUM_MAX; i++)
            {
                x2 = x0 + i;
                y2 = y0 - (float)(xx2[i] * scale);
                g.DrawLine(pen, x1, y1, x2, y2);
                x1 = x2; y1 = y2;
            }
            //凡例
            g.DrawLine(pen, 150, 15, 180, 15);
            g.DrawString("x[m]", new Font("Century", 10), new SolidBrush(Color.Red), 180, 5);
            //zz描画
            pen = new Pen(Color.Black, 2);
            x1 = x0;
            y1 = y0 - (float)(zz2[0] * scale);
            for (i = 1; i < NUM_MAX; i++)
            {
                x2 = x0 + i;
                y2 = y0 - (float)(zz2[i] * scale);
                g.DrawLine(pen, x1, y1, x2, y2);
                x1 = x2; y1 = y2;
            }
            //凡例
            g.DrawLine(pen, 60, 15, 90, 15);
            g.DrawString("z[m]", new Font("Century", 10), new SolidBrush(Color.Black), 90, 5);
            g.DrawString("weight2", new Font("Century", 10), new SolidBrush(Color.Blue), 350, 5);

            g.Dispose();
            picSpace2.Invalidate();

        }

    }
}