﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace CS_Duffing
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }
        //-----------------------------------------------------------------
        const int NUM_MAX = 500;
        double actTime;//実行時間[s]
        double constK1, omega0;
        int numData;

        private void Form1_Load(object sender, EventArgs e)
        {
            cmbTimeStep.Items.Add("0.00001");
            cmbTimeStep.Items.Add("0.00005");
            cmbTimeStep.Items.Add("0.0001");
            cmbTimeStep.Items.Add("0.0005");
            cmbTimeStep.Items.Add("0.001");
            cmbTimeStep.Text = "0.0001";

            cmbActTime.Items.Add("500.0");
            cmbActTime.Items.Add("200.0");
            cmbActTime.Items.Add("100.0");
            cmbActTime.Items.Add("50.0");
            cmbActTime.Items.Add("20.0");
            cmbActTime.Items.Add("10.0");
            cmbActTime.Text = "100.0";

            txtTime0.Text = "0.0";  //グラフ表示の時間始点
            txtConstK1.Text = "1.0";
            txtConstK3.Text = "1.0";
            txtDisp0.Text = "0.8";
            txtEpsilon.Text = "0.1";
            txtForce.Text = "10.0";
            txtAmp1.Text = "10.0"; //波形表示振幅
            txtAmp2.Text = "10.0";//位相表示振幅
            txtOmega.Text = "1.0"; //強制振動角振動数
            picSpace1.Width = NUM_MAX + 75;
            picSpace2.Width = 360;
            picSpace2.Height = 360;

            constK1 = Convert.ToDouble(txtConstK1.Text);
            //共振角周波数
            double omega0 = Math.Sqrt(constK1);
            txtOmega0.Text = Convert.ToString(omega0);
        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            double timeStep = Convert.ToDouble(cmbTimeStep.Text);
            actTime = Convert.ToDouble(cmbActTime.Text);
            constK1 = Convert.ToDouble(txtConstK1.Text);
            double constK3 = Convert.ToDouble(txtConstK3.Text);
            double disp0 = Convert.ToDouble(txtDisp0.Text);
            double force = Convert.ToDouble(txtForce.Text);
            double omega = Convert.ToDouble(txtOmega.Text);//強制振動の角振動数
            double epsilon = Convert.ToDouble(txtEpsilon.Text);
            double time0 = Convert.ToDouble(txtTime0.Text);

            numData = 500;
            //データ取り込み間隔
            int interval = (int)((actTime / timeStep) / NUM_MAX) / (NUM_MAX / numData);

            //共振角周波数
            omega0 = Math.Sqrt(constK1);
            txtOmega0.Text = Convert.ToString(omega0);

            double[] DISP = new double[NUM_MAX + 1];  //変位の確定値
            double[] VEL = new double[NUM_MAX + 1];  //速度の確定値

            //parameters
            double disp = disp0;//変位
            double vel = 0.0;
            double accel;
            double time = 0.0;
            int count = 0;
            int cnt = 0;

            while (time <= time0 + actTime && count < NUM_MAX)
            {
                if (disp > 10.0) disp = 1000.0;
                if (disp < -10.0) disp = -1000.0;
                
                accel = -constK1 * disp - constK3 * disp * disp * disp - epsilon * vel + force * Math.Sin(omega * time);
                vel += accel * timeStep;
                disp += vel * timeStep;

                if (time >= time0 && cnt == 0)
                {//interval回に1回保存
                    DISP[count] = disp;
                    VEL[count] = vel;
                    count++;
                }
                cnt++;
                if (cnt == interval) cnt = 0;

                time += timeStep;

            }
            MessageBox.Show("time = " + time.ToString() + ",  count = " + count.ToString());
            drawWave(DISP, VEL);
            drawPhase(DISP, VEL);
        }

        private void drawWave(double[] DISP, double[] VEL)
        {
            int i;
            float x0, y0, x1, y1, x2, y2;

            float amp = Convert.ToSingle(txtAmp1.Text);
            float scale = (float)(0.4 * picSpace1.Height / amp);

            //左端位置
            x0 = 50.0f;
            y0 = (float)(picSpace1.Height / 2.0f);

            picSpace1.Image = new Bitmap(picSpace1.Width, picSpace1.Height);
            Graphics g = Graphics.FromImage(picSpace1.Image);
            g.Clear(Color.White);

            Pen pen = new Pen(Color.Blue, 1);
            //縦軸
            g.DrawLine(pen, x0, y0 * 1.8f, x0, y0 * 0.2f);
            //横軸

            g.DrawLine(pen, x0, y0, x0 + (float)NUM_MAX, y0);
            //縦軸目盛
            g.DrawLine(pen, x0, y0 - (float)(amp * scale), x0 - 5.0f, y0 - (float)(amp * scale));
            g.DrawString(Convert.ToString(amp), new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 30, y0 - (float)(amp * scale + 10.0));
            g.DrawString("0", new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 20, y0 - 10);
            g.DrawString("-" + Convert.ToString(amp), new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 35, y0 + (float)(amp * scale - 10.0));
            g.DrawLine(pen, x0, y0 + (float)(amp * scale), x0 - 5.0f, y0 + (float)(amp * scale));
            //横軸目盛
            g.DrawLine(pen, x0 + 500, y0, x0 + 500, y0 - 20);
            g.DrawLine(pen, x0 + 250, y0, x0 + 250, y0 - 20);
            //g.DrawString("-" + Convert.ToString(amp), new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 35, y0 + (float)(amp * scale - 10.0));
            if (numData != 500) MessageBox.Show("numData=500としてください");
            if (actTime == 500.0) g.DrawString("500s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 485, y0 + 5);
            else if (actTime == 200.0) g.DrawString("200s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 485, y0 + 5);
            else if (actTime == 100.0) g.DrawString("100s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 480, y0 + 5);
            else if (actTime == 50.0) g.DrawString("50s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 485, y0 + 5);
            else if (actTime == 20.0) g.DrawString("20s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 485, y0 + 5);
            else if (actTime == 10.0) g.DrawString("10s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 485, y0 + 5);

            //変位描画
            pen = new Pen(Color.Black, 2);
            x1 = x0;
            y1 = y0 - (float)(DISP[0] * scale);

            for (i = 1; i < NUM_MAX; i++)
            {
                x2 = x0 + i;
                y2 = y0 - (float)(DISP[i] * scale);
                g.DrawLine(pen, x1, y1, x2, y2);
                x1 = x2; y1 = y2;
            }
            g.DrawLine(pen, 70, 15, 100, 15);
            g.DrawString("disp[m]", new Font("Century", 10), new SolidBrush(Color.Black), 100, 5);

            //速度描画
            pen = new Pen(Color.Red, 2);
            x1 = x0;
            y1 = y0 - (float)(VEL[0] * scale);

            for (i = 1; i < NUM_MAX; i++)
            {
                x2 = x0 + i;
                y2 = y0 - (float)(VEL[i] * scale);
                g.DrawLine(pen, x1, y1, x2, y2);
                x1 = x2; y1 = y2;
            }

            //凡例
            g.DrawLine(pen, 200, 15, 230, 15);
            g.DrawString("vel[m/s]", new Font("Century", 10), new SolidBrush(Color.Red), 230, 5);

            g.Dispose();
            picSpace1.Invalidate();

        }

        private void drawPhase(double[] DISP, double[] VEL)
        {
            int i;
            float x0, y0, x1, y1, x2, y2;

            float amp = Convert.ToSingle(txtAmp2.Text);
            float scaleX = (float)(0.4 * picSpace2.Width / amp);
            float scaleY = (float)(0.4 * picSpace2.Height / amp);

            //中心位置
            x0 = picSpace2.Width / 2.0f;
            y0 = (float)(picSpace2.Height / 2.0f);

            picSpace2.Image = new Bitmap(picSpace2.Width, picSpace2.Height);
            Graphics g = Graphics.FromImage(picSpace2.Image);
            g.Clear(Color.White);

            Pen pen = new Pen(Color.Blue, 1);
            //縦軸
            g.DrawLine(pen, x0, y0 * 1.8f, x0, y0 * 0.2f);
            //横軸
            g.DrawLine(pen, x0-180.0f, y0, x0 + 180.0f, y0);

            //縦軸目盛
            g.DrawLine(pen, x0-5.0f, y0 - (float)(amp * scaleY), x0 + 5.0f, y0 - (float)(amp * scaleY));
            g.DrawString(Convert.ToString(amp), new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 30, y0 - (float)(amp * scaleY + 10.0));
            g.DrawString("0", new Font("Century", 10), new SolidBrush(Color.Blue), x0 -12, y0 + 3);
            g.DrawString("-" + Convert.ToString(amp), new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 35, y0 + (float)(amp * scaleY - 10.0));
            g.DrawLine(pen, x0 - 5.0f, y0 + (float)(amp * scaleY), x0 + 5.0f, y0 + (float)(amp * scaleY));
            g.DrawString("vel", new Font("Century", 10), new SolidBrush(Color.Blue), picSpace2.Width / 2.0f - 10.0f, 15.0f);

            //横軸目盛
            g.DrawLine(pen, x0 + (float)(amp * scaleX), y0 - 5.0f, x0 + (float)(amp * scaleX), y0 + 5.0f);
            g.DrawString(Convert.ToString(amp), new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 8 + (float)(amp * scaleX), y0 + 5.0f);
            g.DrawLine(pen, x0 - (float)(amp * scaleX), y0 - 5.0f, x0 - (float)(amp * scaleX), y0 + 5.0f);
            g.DrawString("-" + Convert.ToString(amp), new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 12 - (float)(amp * scaleX), y0 + 5.0f);
            g.DrawString("disp", new Font("Century", 10), new SolidBrush(Color.Blue), picSpace2.Width -60.0f, picSpace2.Height / 2.0f + 20.0f);
           
            //位相描画
            pen = new Pen(Color.Black, 2);
            x1 = x0 + (float)(DISP[0] * scaleX);//変位
            y1 = y0 - (float)(VEL[0] * scaleY); //速度

            for (i = 1; i < NUM_MAX; i++)
            {
                x2 = x0 + (float)(DISP[i] * scaleX); 
                y2 = y0 - (float)(VEL[i] * scaleY);
                g.DrawLine(pen, x1, y1, x2, y2);
                x1 = x2; y1 = y2;
            }
            g.DrawString("phase", new Font("Century", 10), new SolidBrush(Color.Black), 20, 5);

            g.Dispose();
            picSpace2.Invalidate();
        }

        private void txtConstK1_TextChanged(object sender, EventArgs e)
        {
            if (txtConstK1.Text == "") return;
            constK1 = Convert.ToDouble(txtConstK1.Text);
            //共振角周波数
            omega0 = Math.Sqrt(constK1);
            txtOmega0.Text = Convert.ToString(omega0);

        }

        private void txtConstK3_TextChanged(object sender, EventArgs e)
        {
            if (txtConstK3.Text == "") return;

        }
    }
}