﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace CS_Spring0
{
  public partial class Form1 : Form
  {
    public Form1()
    {
      InitializeComponent();
    }
//-----------------------------------------------------------------
    const int NUM_MAX = 500;
    double actTime;//実行時間[s]
    double mass, constK, omega0;
    int numData;

    private void Form1_Load(object sender, EventArgs e)
    {
      cmbTimeStep.Items.Add("0.0001");
      cmbTimeStep.Items.Add("0.005");
      cmbTimeStep.Items.Add("0.001");
      cmbTimeStep.Items.Add("0.005");
      cmbTimeStep.Items.Add("0.01");
      cmbTimeStep.Text = "0.001";

      cmbActTime.Items.Add("100.0");
      cmbActTime.Items.Add("50.0");
      cmbActTime.Items.Add("25.0");
      cmbActTime.Items.Add("10.0");
      cmbActTime.Items.Add("5.0");
      cmbActTime.Text = "5.0";

      cmbMass.Items.Add("0.01");
      cmbMass.Items.Add("0.05");
      cmbMass.Items.Add("0.1");
      cmbMass.Items.Add("0.5");
      cmbMass.Items.Add("1.0");
      cmbMass.Items.Add("5.0");
      cmbMass.Items.Add("10.0");
      cmbMass.Text = "0.1";

      cmbNumData.Items.Add("500");
      cmbNumData.Text = "500";

      txtConstK.Text = "10.0";
      txtLength0.Text = "1.0";
      txtDisp0.Text = "0.8";
      txtGamma0.Text = "0.5";
      txtGamma1.Text = "10.0";
      txtGamma2.Text = "50.0";
      txtAmplitude.Text = "1.0";
      picSpace.Width = NUM_MAX + 100;

      mass = Convert.ToDouble(cmbMass.Text);
      constK = Convert.ToDouble(txtConstK.Text);
      //共振角周波数
      double omega0 = Math.Sqrt(constK / mass);
      txtOmega0.Text = Convert.ToString(omega0);
    }

    private void btnStart_Click(object sender, EventArgs e)
    {
      double timeStep = Convert.ToDouble(cmbTimeStep.Text);
      actTime = Convert.ToDouble(cmbActTime.Text);
      mass = Convert.ToDouble(cmbMass.Text);
      constK = Convert.ToDouble(txtConstK.Text);
      double length0 = Convert.ToDouble(txtLength0.Text);
      double disp0 = Convert.ToDouble(txtDisp0.Text);
      double[] gamma = new double[3];
      gamma[0] = Convert.ToDouble(txtGamma0.Text);
      gamma[1] = Convert.ToDouble(txtGamma1.Text);
      gamma[2] = Convert.ToDouble(txtGamma2.Text);

      numData = Convert.ToInt32(cmbNumData.Text);
      //データ取り込み間隔
      int interval = (int)((actTime / timeStep) / NUM_MAX) / (NUM_MAX / numData);

      //共振角周波数
      omega0 = Math.Sqrt(constK / mass);
      txtOmega0.Text = Convert.ToString(omega0);

      double disp;//変位
      double length = length0 + disp0;
      double vel = 0.0;
      double posX = length;//一端は原点に固定
      double accel;
      double time = 0.0;
      int count = 0;
      int cnt = 0;
      double[,] ratio = new double[3,NUM_MAX + 1];  //変位率の確定値

      for (int j = 0; j < 3; j++)
      {
        length = length0 + disp0;
        vel = 0.0;
        posX = length;//一端は原点に固定
        time = 0.0;
        count = 0;
        cnt = 0;
        while (time <= actTime && count < NUM_MAX)
        {
          disp = length - length0;
          accel = -constK * disp / mass - 2.0 * gamma[j] * vel;
          vel += accel * timeStep;
          posX += vel * timeStep;

          length = posX;

          if (cnt == 0)
          {//interval回に1回保存
            ratio[j, count] = disp / length0;
            count++;
          }
          cnt++;
          if (cnt == interval) cnt = 0;

          time += timeStep;
        }
      }
      drawGraph(gamma, ratio);
    }
    private void drawGraph(double[] gamma, double[,] ratio)
    {
      int i;
      float x0, y0, x1, y1, x2, y2;

      float amp = Convert.ToSingle(txtAmplitude.Text);
      float scale = (float)(0.4 * picSpace.Height / amp);//posの倍率

      //左端位置
      x0 = 50.0f;
      y0 = (float)(picSpace.Height / 2.0f);

      picSpace.Image = new Bitmap(picSpace.Width, picSpace.Height);
      Graphics g = Graphics.FromImage(picSpace.Image);
      g.Clear(Color.White);

      Pen pen = new Pen(Color.Blue, 1);
      //縦軸
      g.DrawLine(pen, x0, y0 * 1.8f, x0, y0 * 0.2f);
      //横軸
      g.DrawLine(pen, x0, y0, x0 + (float)NUM_MAX, y0);
      //縦軸目盛
      g.DrawLine(pen, x0, y0 - (float)(amp * scale), x0 - 5.0f, y0 - (float)(amp * scale));
      g.DrawString(Convert.ToString(amp), new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 30, y0 - (float)(amp * scale + 10.0));
      g.DrawString("0", new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 20, y0 - 10);
      g.DrawString("-" + Convert.ToString(amp), new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 35, y0 + (float)(amp * scale - 10.0));
      g.DrawLine(pen, x0, y0 + (float)(amp * scale), x0 - 5.0f, y0 + (float)(amp * scale));
      //横軸目盛
      g.DrawLine(pen, x0 + 500, y0, x0 + 500, y0 - 20);
      g.DrawLine(pen, x0 + 250, y0, x0 + 250, y0 - 20);
      //g.DrawString("-" + Convert.ToString(amp), new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 35, y0 + (float)(amp * scale - 10.0));
      if (numData != 500) MessageBox.Show("numData=500としてください");
      if (actTime == 100.0) g.DrawString("100s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 480, y0 + 5);
      else if (actTime == 50.0) g.DrawString("50s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 485, y0 + 5);
      else if (actTime == 25.0) g.DrawString("25s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 485, y0 + 5);
      else if (actTime == 10.0) g.DrawString("10s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 485, y0 + 5);
      else if (actTime == 5.0) g.DrawString("5s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 490, y0 + 5);

      //変位率（ratio)描画
      //for gamma[0]
      pen = new Pen(Color.Black, 2);
      x1 = x0;
      y1 = y0 - (float)(ratio[0,0] * scale);

      for (i = 1; i < NUM_MAX; i++)
      {
        x2 = x0 + i;
        y2 = y0 - (float)(ratio[0,i] * scale);
        g.DrawLine(pen, x1, y1, x2, y2);
        x1 = x2; y1 = y2;
      }
      //凡例
      g.DrawLine(pen, 60, 15, 90, 15);
      g.DrawString("gamma="+gamma[0].ToString(), new Font("Century", 10), new SolidBrush(Color.Black), 90, 5);

      //for gamma[1]
      pen = new Pen(Color.Red, 2);
      x1 = x0;
      y1 = y0 - (float)(ratio[1, 0] * scale);

      for (i = 1; i < NUM_MAX; i++)
      {
        x2 = x0 + i;
        y2 = y0 - (float)(ratio[1, i] * scale);
        g.DrawLine(pen, x1, y1, x2, y2);
        x1 = x2; y1 = y2;
      }
      //凡例
      g.DrawLine(pen, 210, 15, 240, 15);
      g.DrawString("gamma=" + gamma[1].ToString(), new Font("Century", 10), new SolidBrush(Color.Red), 240, 5);

      //for gamma[2]
      pen = new Pen(Color.Blue, 2);
      x1 = x0;
      y1 = y0 - (float)(ratio[2, 0] * scale);

      for (i = 1; i < NUM_MAX; i++)
      {
        x2 = x0 + i;
        y2 = y0 - (float)(ratio[2, i] * scale);
        g.DrawLine(pen, x1, y1, x2, y2);
        x1 = x2; y1 = y2;
      }
      //凡例
      g.DrawLine(pen, 350, 15, 380, 15);
      g.DrawString("gamma=" + gamma[2].ToString(), new Font("Century", 10), new SolidBrush(Color.Blue), 380, 5);

      g.Dispose();
      picSpace.Invalidate();

    }

    private void cmbMass_SelectedIndexChanged(object sender, EventArgs e)
    {
      mass = Convert.ToDouble(cmbMass.Text);
      //共振角周波数
      omega0 = Math.Sqrt(constK / mass);
      txtOmega0.Text = Convert.ToString(omega0);
    }

    private void txtConstK_TextChanged(object sender, EventArgs e)
    {
      if (txtConstK.Text == "") return;
      constK = Convert.ToDouble(txtConstK.Text);
      //共振角周波数
      omega0 = Math.Sqrt(constK / mass);
      txtOmega0.Text = Convert.ToString(omega0);
    }
    //---------------------------------------------------------------------------------------

  }
}