﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace CS_SpringPendulum12
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }
        //--------------------------------------------------------------------------
        const int NUM_MAX = 500;
        double actTime;//実行時間[s]
        int numData;
        string method;

        private void Form1_Load(object sender, EventArgs e)
        {
            cmbTimeStep.Items.Add("0.00001");
            cmbTimeStep.Items.Add("0.0001");
            cmbTimeStep.Items.Add("0.001");
            cmbTimeStep.Items.Add("0.01");
            cmbTimeStep.Text = "0.0001";

            cmbActTime.Items.Add("100.0");
            cmbActTime.Items.Add("50.0");
            cmbActTime.Items.Add("25.0");
            cmbActTime.Items.Add("10.0");
            cmbActTime.Items.Add("5.0");
            cmbActTime.Text = ("10.0");

            listMethod.Items.Add("Lagrange");
            listMethod.Items.Add("Easy");
            listMethod.Text = "Lagrange";

            cmbNumData.Items.Add("500");
            cmbNumData.Text = "500";

            txtTime0.Text = "0.0"; //グラフ表示の時間始点
            txtMass.Text = "0.1";  //おもりの質量
            txtLength.Text = "1.0";//糸の長さ
            txtTheta.Text = "50.0";//振れ角
            txtConstK.Text = "10.0";//ばね定数
            txtDisp.Text = "0.2";
            txtViscous.Text = "0.0";//粘性抵抗係数
            txtAmplitude.Text = "1.0";

            picSpace1.Width = NUM_MAX + 80;
            picSpace2.Width = NUM_MAX + 80;
        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            double timeStep = Convert.ToDouble(cmbTimeStep.Text);
            actTime = Convert.ToDouble(cmbActTime.Text);
            double mass = Convert.ToDouble(txtMass.Text);
            double length0 = Convert.ToDouble(txtLength.Text);
            double theta0 = Convert.ToDouble(txtTheta.Text);
            double constK = Convert.ToDouble(txtConstK.Text);
            double disp0 = Convert.ToDouble(txtDisp.Text);
            double viscous = Convert.ToDouble(txtViscous.Text);
            double time0 = Convert.ToDouble(txtTime0.Text);
            method = listMethod.Text;
          
            numData = Convert.ToInt32(cmbNumData.Text);
            //データ取り込み間隔
            int interval = (int)((actTime / timeStep) / NUM_MAX) / (NUM_MAX / numData);

            //固有周期
            double omega0 = Math.Sqrt(constK / mass);
            double period0 = 2.0 * Math.PI / omega0;
            txtPeriod.Text = period0.ToString("0.000");//固有周期の表示 

            //Lagrangeで使用する変数
            double alpha, beta, theta;
            double accel, vel, len, disp;
            //簡易解法
            double posX, posZ, velX, velZ, accX, accZ;

            double g = 9.8;

            float[] xx = new float[NUM_MAX + 1];//おもりのy座標
            float[] zz = new float[NUM_MAX + 1];//おもりのｚ座標（おもり1の自然の位置を0）

            //初期値(Lagrange)
            theta = theta0 * Math.PI / 180.0;
            beta = 0.0;
            len = length0 + disp0;
            vel = 0.0;
            //初期値(簡易解法）
            theta = theta0 * Math.PI / 180.0;
            len = length0 + disp0;
            posX = len * Math.Sin(theta);
            posZ = length0 - len * Math.Cos(theta);
            velX = velZ = 0.0;

            double time = 0.0;
            double ss, cc;
            int count = 0;
            int cnt = 0;

            if (method == "Lagrange")
            {
                while (time <= time0 + actTime && count < NUM_MAX)
                {
                    //Euler(Lagrange)
                    ss = Math.Sin(theta);
                    cc = Math.Cos(theta);
                    disp = len - length0;

                    accel = len * beta * beta + g * cc - constK * disp / mass - (viscous / mass) * vel;
                    alpha = -(2.0 * vel * beta + g * ss) / len - (viscous / mass) * beta;

                    vel += accel * timeStep;
                    beta += alpha * timeStep;
                    len += vel * timeStep;
                    theta += beta * timeStep;

                    if (time >= time0 && cnt == 0)
                    {//interval回に1回保存
                        xx[count] = (float)(len * ss);
                        zz[count] = (float)(length0 - len * cc);//鉛直軸自然長の位置を基準
                        count++;
                    }
                    cnt++;
                    if (cnt == interval) cnt = 0;

                    time += timeStep;
                }
                MessageBox.Show("time = " + time.ToString() + ",  count = " + count.ToString() + ",  interval= " + interval.ToString());

                drawGraphL(xx, zz);
            }

            else
            {
                while (time <= time0 + actTime && count < NUM_MAX)
                {
                    //Euler(簡易解法)
                    ss = Math.Sin(theta);
                    cc = Math.Cos(theta);
                    disp = len - length0;

                    accX = -constK * disp * ss / mass - (viscous / mass) * velX;
                    accZ = constK * disp * cc / mass - g - (viscous / mass) * velZ;

                    velX += accX * timeStep;
                    velZ += accZ * timeStep;
                    posX += velX * timeStep;
                    posZ += velZ * timeStep;
                    len = Math.Sqrt(posX * posX + (length0 - posZ) * (length0 - posZ));
                    theta = Math.Atan2(posX, length0 - posZ);

                    if (time >= time0 && cnt == 0)
                    {//interval回に1回保存
                        xx[count] = (float)(posX);
                        zz[count] = (float)(posZ);//鉛直軸自然長の位置を基準
                        count++;
                    }
                    cnt++;
                    if (cnt == interval) cnt = 0;

                    time += timeStep;
                }
                MessageBox.Show("time = " + time.ToString() + ",  count = " + count.ToString() + ",  interval= " + interval.ToString());
                drawGraphE(xx, zz);
            }
        }

        private void drawGraphL(float[] xx, float[] zz)
        {
            int i;
            float x0, y0, x1, y1, x2, y2;

            double amp = Convert.ToDouble(txtAmplitude.Text);
            double scale = 0.4 * picSpace1.Height / amp;//倍率

            //Lagrange
            //左端位置
            x0 = 50.0f;
            y0 = (float)(picSpace1.Height / 2.0f);

            Graphics g;
            picSpace1.Image = new Bitmap(picSpace1.Width, picSpace1.Height);
            g = Graphics.FromImage(picSpace1.Image);
            g.Clear(Color.White);

            Pen pen = new Pen(Color.Blue, 1);
            //縦軸
            g.DrawLine(pen, x0, y0 * 1.8f, x0, y0 * 0.2f);
            //横軸
            g.DrawLine(pen, x0, y0, x0 + (float)NUM_MAX, y0);
            //縦軸目盛
            g.DrawLine(pen, x0, y0 - (float)(amp * scale), x0 - 5.0f, y0 - (float)(amp * scale));
            g.DrawString(Convert.ToString(amp), new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 30, y0 - (float)(amp * scale + 10.0));
            g.DrawString("0", new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 20, y0 - 10);
            g.DrawString("-" + Convert.ToString(amp), new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 35, y0 + (float)(amp * scale - 10.0));
            g.DrawLine(pen, x0, y0 + (float)(amp * scale), x0 - 5.0f, y0 + (float)(amp * scale));
            //横軸目盛
            g.DrawLine(pen, x0 + 500, y0, x0 + 500, y0 - 20);
            g.DrawLine(pen, x0 + 250, y0, x0 + 250, y0 - 20);

            if (numData != 500) MessageBox.Show("numData=500としてください");
            if (actTime == 100.0) g.DrawString("100s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 480, y0 + 5);
            else if (actTime == 50.0) g.DrawString("50s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 480, y0 + 5);
            else if (actTime == 10.0) g.DrawString("10s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 480, y0 + 5);

            //xx描画
            pen = new Pen(Color.Red, 2);
            x1 = x0;
            y1 = y0 - (float)(xx[0] * scale);
            for (i = 1; i < NUM_MAX; i++)
            {
                x2 = x0 + i;
                y2 = y0 - (float)(xx[i] * scale);
                g.DrawLine(pen, x1, y1, x2, y2);
                x1 = x2; y1 = y2;
            }
            //凡例
            g.DrawLine(pen, 150, 15, 180, 15);
            g.DrawString("x[m]", new Font("Century", 10), new SolidBrush(Color.Red), 180, 5);
            //zz描画
            pen = new Pen(Color.Black, 2);
            x1 = x0;
            y1 = y0 - (float)(zz[0] * scale);
            for (i = 1; i < NUM_MAX; i++)
            {
                x2 = x0 + i;
                y2 = y0 - (float)(zz[i] * scale);
                g.DrawLine(pen, x1, y1, x2, y2);
                x1 = x2; y1 = y2;
            }
            //凡例
            g.DrawLine(pen, 60, 15, 90, 15);
            g.DrawString("z[m]", new Font("Century", 10), new SolidBrush(Color.Black), 90, 5);
            g.DrawString("Lagrange", new Font("Century", 10), new SolidBrush(Color.Blue), 350, 5);
            g.Dispose();
            picSpace1.Invalidate();
        }

        private void drawGraphE(float[] xx, float[] zz)
        {
            int i;
            float x0, y0, x1, y1, x2, y2;

            double amp = Convert.ToDouble(txtAmplitude.Text);
            double scale = 0.4 * picSpace1.Height / amp;//倍率
            //簡易解法
            //左端位置
            x0 = 50.0f;
            y0 = (float)(picSpace2.Height / 2.0f);

            Graphics g;
            picSpace2.Image = new Bitmap(picSpace2.Width, picSpace2.Height);
            g = Graphics.FromImage(picSpace2.Image);
            g.Clear(Color.White);

            Pen pen = new Pen(Color.Blue, 1);
            //縦軸
            g.DrawLine(pen, x0, y0 * 1.8f, x0, y0 * 0.2f);
            //横軸
            g.DrawLine(pen, x0, y0, x0 + (float)NUM_MAX, y0);
            //縦軸目盛
            g.DrawLine(pen, x0, y0 - (float)(amp * scale), x0 - 5.0f, y0 - (float)(amp * scale));
            g.DrawString(Convert.ToString(amp), new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 30, y0 - (float)(amp * scale + 10.0));
            g.DrawString("0", new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 20, y0 - 10);
            g.DrawString("-" + Convert.ToString(amp), new Font("Century", 10), new SolidBrush(Color.Blue), x0 - 35, y0 + (float)(amp * scale - 10.0));
            g.DrawLine(pen, x0, y0 + (float)(amp * scale), x0 - 5.0f, y0 + (float)(amp * scale));
            //横軸目盛
            g.DrawLine(pen, x0 + 500, y0, x0 + 500, y0 - 20);
            g.DrawLine(pen, x0 + 250, y0, x0 + 250, y0 - 20);

            if (numData != 500) MessageBox.Show("numData=500としてください");
            if (actTime == 100.0) g.DrawString("100s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 480, y0 + 5);
            else if (actTime == 50.0) g.DrawString("50s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 480, y0 + 5);
            else if (actTime == 10.0) g.DrawString("10s", new Font("Century", 10), new SolidBrush(Color.Blue), x0 + 480, y0 + 5);

            //xx描画
            pen = new Pen(Color.Red, 2);
            x1 = x0;
            y1 = y0 - (float)(xx[0] * scale);
            for (i = 1; i < NUM_MAX; i++)
            {
                x2 = x0 + i;
                y2 = y0 - (float)(xx[i] * scale);
                g.DrawLine(pen, x1, y1, x2, y2);
                x1 = x2; y1 = y2;
            }
            //凡例
            g.DrawLine(pen, 150, 15, 180, 15);
            g.DrawString("x[m]", new Font("Century", 10), new SolidBrush(Color.Red), 180, 5);
            //zz描画
            pen = new Pen(Color.Black, 2);
            x1 = x0;
            y1 = y0 - (float)(zz[0] * scale);
            for (i = 1; i < NUM_MAX; i++)
            {
                x2 = x0 + i;
                y2 = y0 - (float)(zz[i] * scale);
                g.DrawLine(pen, x1, y1, x2, y2);
                x1 = x2; y1 = y2;
            }
            //凡例
            g.DrawLine(pen, 60, 15, 90, 15);
            g.DrawString("z[m]", new Font("Century", 10), new SolidBrush(Color.Black), 90, 5);
            g.DrawString("Easy", new Font("Century", 10), new SolidBrush(Color.Blue), 350, 5);

            g.Dispose();
            picSpace2.Invalidate();

        }
    }
}